@@ -15,6 +15,8 @@ from collections import namedtuple, deque
 from xml.sax.saxutils import escape
 from distutils import version
 from email.parser import HeaderParser
+import urllib.request
+import xmlrpc.client
 
 import pkg_resources
 
@@ -23,18 +25,22 @@ try:
 except ImportError:
     docutils = None
 
-from PyQt4.QtGui import (
+from AnyQt.QtWidgets import (
     QWidget, QDialog, QLabel, QLineEdit, QTreeView, QHeaderView,
-    QTextBrowser, QTextOption, QDialogButtonBox, QProgressDialog,
-    QVBoxLayout, QPalette, QStandardItemModel, QStandardItem,
-    QSortFilterProxyModel, QItemSelectionModel, QStyle, QStyledItemDelegate,
-    QStyleOptionViewItemV4, QApplication, QHBoxLayout
+    QTextBrowser, QDialogButtonBox, QProgressDialog,
+    QVBoxLayout, QStyle, QStyledItemDelegate, QStyleOptionViewItem,
+    QApplication, QHBoxLayout
 )
 
-from PyQt4.QtCore import (
+from AnyQt.QtGui import (
+    QStandardItemModel, QStandardItem, QPalette, QTextOption
+)
+
+from AnyQt.QtCore import (
+    QSortFilterProxyModel, QItemSelectionModel,
     Qt, QObject, QMetaObject, QEvent, QSize, QTimer, QThread, Q_ARG
 )
-from PyQt4.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
+from AnyQt.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
 
 from ..config import ADDON_KEYWORD
 from ..gui.utils import message_warning, message_information, \
@@ -123,7 +129,7 @@ class TristateCheckItemDelegate(QStyledItemDelegate):
         if event.type() in {QEvent.MouseButtonPress, QEvent.MouseButtonRelease,
                             QEvent.MouseButtonDblClick}:
             pos = event.pos()
-            opt = QStyleOptionViewItemV4(option)
+            opt = QStyleOptionViewItem(option)
             self.initStyleOption(opt, index)
             rect = style.subElementRect(
                 QStyle.SE_ItemViewItemCheckIndicator, opt, widget)
@@ -220,8 +226,8 @@ class AddonManagerWidget(QWidget):
             self.__update_details
         )
         header = self.__view.header()
-        header.setResizeMode(0, QHeaderView.Fixed)
-        header.setResizeMode(2, QHeaderView.ResizeToContents)
+        header.setSectionResizeMode(0, QHeaderView.Fixed)
+        header.setSectionResizeMode(2, QHeaderView.ResizeToContents)
 
         self.__details = QTextBrowser(
             frameShape=QTextBrowser.NoFrame,
@@ -333,14 +339,10 @@ class AddonManagerWidget(QWidget):
 
     def __data_changed(self, topleft, bottomright):
         rows = range(topleft.row(), bottomright.row() + 1)
-        proxy = self.__view.model()
-        map_to_source = proxy.mapToSource
-
         for i in rows:
-            sourceind = map_to_source(proxy.index(i, 0))
-            modelitem = self.__model.itemFromIndex(sourceind)
-            actionitem = self.__model.item(modelitem.row(), 3)
-            item = self.__items[modelitem.row()]
+            modelitem = self.__model.item(i, 0)
+            actionitem = self.__model.item(i, 3)
+            item = self.__items[i]
 
             state = modelitem.checkState()
             flags = modelitem.flags()
@@ -629,14 +631,26 @@ class AddonManagerDialog(QDialog):
         self.accept()
 
 
+class SafeUrllibTransport(xmlrpc.client.Transport):
+    """Urllib for HTTPS connections that automatically handles proxies."""
+
+    def single_request(self, host, handler, request_body, verbose=False):
+        req = urllib.request.Request('https://%s%s' % (host, handler), request_body)
+        req.add_header('User-agent', self.user_agent)
+        req.add_header('Content-Type', 'text/xml')
+        self.verbose = verbose
+        opener = urllib.request.build_opener()
+        return self.parse_response(opener.open(req))
+
+
 def list_pypi_addons():
     """
     List add-ons available on pypi.
     """
     from ..config import ADDON_PYPI_SEARCH_SPEC
-    import xmlrpc.client
+
     pypi = xmlrpc.client.ServerProxy(
-        "https://pypi.python.org/pypi",
+        "https://pypi.python.org/pypi/",
         transport=xmlrpc.client.SafeTransport()
     )
     addons = pypi.search(ADDON_PYPI_SEARCH_SPEC)
@@ -701,6 +715,20 @@ def unique(iterable):
     return (el for el in iterable if not observed(el))
 
 
+def _env_with_proxies():
+    """
+    Return system environment with proxies obtained from urllib so that
+    they can be used with pip.
+    """
+    proxies = urllib.request.getproxies()
+    env = dict(os.environ)
+    if "http" in proxies:
+        env["HTTP_PROXY"] = proxies["http"]
+    if "https" in proxies:
+        env["HTTPS_PROXY"] = proxies["https"]
+    return env
+
+
 Install, Upgrade, Uninstall = 1, 2, 3
 
 
@@ -740,7 +768,7 @@ class Installer(QObject):
             cmd = (["-m", "pip", "install"] +
                    (["--user"] if self.__user_install else []) +
                    [inst_name])
-            process = python_process(cmd, bufsize=-1, universal_newlines=True)
+            process = python_process(cmd, bufsize=-1, universal_newlines=True, env=_env_with_proxies())
             retcode, output = self.__subprocessrun(process)
 
             if retcode != 0:
@@ -755,7 +783,7 @@ class Installer(QObject):
             cmd = (["-m", "pip", "install", "--upgrade", "--no-deps"] +
                    (["--user"] if self.__user_install else []) +
                    [inst_name])
-            process = python_process(cmd, bufsize=-1, universal_newlines=True)
+            process = python_process(cmd, bufsize=-1, universal_newlines=True, env=_env_with_proxies())
             retcode, output = self.__subprocessrun(process)
 
             if retcode != 0:
@@ -766,7 +794,7 @@ class Installer(QObject):
             cmd = (["-m", "pip", "install"] +
                    (["--user"] if self.__user_install else []) +
                    [inst_name])
-            process = python_process(cmd, bufsize=-1, universal_newlines=True)
+            process = python_process(cmd, bufsize=-1, universal_newlines=True, env=_env_with_proxies())
             retcode, output = self.__subprocessrun(process)
 
             if retcode != 0:
@@ -778,7 +806,7 @@ class Installer(QObject):
             self.setStatusMessage("Uninstalling {}".format(dist.project_name))
 
             cmd = ["-m", "pip", "uninstall", "--yes", dist.project_name]
-            process = python_process(cmd, bufsize=-1, universal_newlines=True)
+            process = python_process(cmd, bufsize=-1, universal_newlines=True, env=_env_with_proxies())
             retcode, output = self.__subprocessrun(process)
 
             if self.__user_install:
