@@ -283,15 +283,30 @@ class FileFormat(metaclass=FileFormatMeta):
 
     @classmethod
     def write_table_metadata(cls, filename, data):
-        if isinstance(filename, str) and getattr(data, 'attributes', {}):
-            with open(filename + '.metadata', 'wb') as f:
-                pickle.dump(data.attributes, f, pickle.HIGHEST_PROTOCOL)
+        if isinstance(filename, str) and hasattr(data, 'attributes'):
+            if all(isinstance(key, str) and isinstance(value, str)
+                   for key, value in data.attributes.items()):
+                with open(filename + '.metadata', 'w') as f:
+                    f.write("\n".join("{}: {}".format(*kv)
+                                      for kv in data.attributes.items()))
+            else:
+                with open(filename + '.metadata', 'wb') as f:
+                    pickle.dump(data.attributes, f, pickle.HIGHEST_PROTOCOL)
 
     @classmethod
     def set_table_metadata(cls, filename, table):
+        # pylint: disable=bare-except
         if isinstance(filename, str) and path.exists(filename + '.metadata'):
-            with open(filename + '.metadata', 'rb') as f:
-                table.attributes = pickle.load(f)
+            try:
+                with open(filename + '.metadata', 'rb') as f:
+                    table.attributes = pickle.load(f)
+            # Unpickling throws different exceptions, not just UnpickleError
+            except:
+                with open(filename + '.metadata') as f:
+                    table.attributes = OrderedDict(
+                        (k.strip(), v.strip())
+                        for k, v in (line.split(":", 1)
+                                     for line in f.readlines()))
 
     @classmethod
     def locate(cls, filename, search_dirs=('.',)):
@@ -555,6 +570,7 @@ class FileFormat(metaclass=FileFormatMeta):
 
             cols, domain_vars = append_to
             cols.append(col)
+            var = None
             if domain_vars is not None:
                 if names and names[col]:
                     # Use existing variable if available
@@ -579,10 +595,11 @@ class FileFormat(metaclass=FileFormatMeta):
                                 continue
                             bn.replace(column, offset + oldval, new_order.index(val))
 
-            if coltype is TimeVariable:
+            if isinstance(var, TimeVariable) or coltype is TimeVariable:
                 # Re-parse the values because only now after coltype.make call
                 # above, variable var is the correct one
-                values = [var.parse(i) for i in orig_values]
+                _var = var if isinstance(var, TimeVariable) else TimeVariable('_')
+                values = [_var.parse(i) for i in orig_values]
 
             # Write back the changed data. This is needeed to pass the
             # correct, converted values into Table.from_numpy below
