@@ -8,12 +8,12 @@ from math import isnan
 import random
 
 from unittest.mock import Mock, MagicMock, patch
-from scipy.sparse import csr_matrix, issparse
+import scipy.sparse as sp
 import numpy as np
 
 from Orange import data
-from Orange.data import filter, Variable
-from Orange.data import Unknown
+from Orange.data import (filter, Unknown, Variable, Table, DiscreteVariable,
+                         ContinuousVariable, Domain, StringVariable)
 from Orange.tests import test_dirname
 
 
@@ -651,7 +651,7 @@ class TableTestCase(unittest.TestCase):
 
     def test_copy_sparse(self):
         t = data.Table('iris')
-        t.X = csr_matrix(t.X)
+        t.X = sp.csr_matrix(t.X)
         copy = t.copy()
 
         self.assertEqual((t.X != copy.X).nnz, 0)      # sparse matrices match by content
@@ -1022,6 +1022,12 @@ class TableTestCase(unittest.TestCase):
         x = filter.Values([f])(d)
         self.assertEqual(len(x), 7)
 
+    def test_valueFilter_discrete_meta_is_defined(self):
+        d = data.Table("test9.tab")
+        f = filter.FilterDiscrete(-4, None)
+        x = filter.Values([f])(d)
+        self.assertEqual(len(x), 8)
+
     def test_valueFilter_string_case_sens(self):
         d = data.Table("zoo")
         col = d[:, "name"].metas[:, 0]
@@ -1192,8 +1198,7 @@ class TableTestCase(unittest.TestCase):
 
     def test_attributes(self):
         table = data.Table("iris")
-        self.assertEqual(table.attributes, {})
-        table.attributes[1] = "test"
+        table.attributes = {1: "test"}
         table2 = table[:4]
         self.assertEqual(table2.attributes[1], "test")
         table2.attributes[1] = "modified"
@@ -1313,8 +1318,6 @@ class CreateTableWithUrl(TableTests):
     class _MockUrlOpen(MagicMock):
         headers = {'content-disposition': 'attachment; filename="Something-FormResponses.tsv"; '
                                           'filename*=UTF-8''Something%20%28Responses%29.tsv'}
-        url = 'https://docs.google.com/spreadsheets/d/ABCD/edit'
-
         def __enter__(self): return self
 
         def __exit__(self, *args, **kwargs): pass
@@ -1327,12 +1330,16 @@ a\tb\tc
     urlopen = _MockUrlOpen()
 
     @patch('Orange.data.io.urlopen', urlopen)
-    def test_google_sheets(self):
-        d = data.Table(self.urlopen.url)
-        self.urlopen.assert_called_with('https://docs.google.com/spreadsheets/d/ABCD/export?format=tsv',
-                                        timeout=10)
-        self.assertEqual(len(d), 2)
-        self.assertEqual(d.name, 'Something-FormResponses')
+    def test_trimmed_urls(self):
+        for url in ('https://docs.google.com/spreadsheets/d/ABCD/edit',
+                    'https://www.dropbox.com/s/ABCD/filename.csv'):
+            self._MockUrlOpen.url = url
+            d = data.Table(url)
+            request = self.urlopen.call_args[0][0]
+            self.assertNotEqual(url, request.full_url)
+            self.assertIn('Mozilla/5.0', request.headers.get('User-agent', ''))
+            self.assertEqual(len(d), 2)
+            self.assertEqual(d.name, 'Something-FormResponses')
 
 
 class CreateTableWithDomain(TableTests):
@@ -1747,23 +1754,68 @@ class CreateTableWithDomainAndTable(TableTests):
         self.assert_table_with_filter_matches(
             new_table, self.table[:0], xcols=order, ycols=order, mcols=order)
 
-    def test_from_table_on_sparse_data(self):
+    def test_from_table_sparse_move_some_to_empty_metas(self):
         iris = data.Table("iris")
-        iris.X = csr_matrix(iris.X)
-
+        iris.X = sp.csr_matrix(iris.X)
         new_domain = data.domain.Domain(iris.domain.attributes[:2], iris.domain.class_vars,
-                                        iris.domain.metas, source=iris.domain)
+                                        iris.domain.attributes[2:], source=iris.domain)
         new_iris = data.Table.from_table(new_domain, iris)
-        self.assertTrue(issparse(new_iris.X))
-        self.assertEqual(new_iris.X.shape[1], 2)
-        self.assertEqual(len(new_iris.domain.attributes), 2)
 
-        all_vars = chain(iris.domain.variables, iris.domain.metas)
-        n_all = len(iris.domain) + len(iris.domain.metas)
-        new_domain = data.domain.Domain([], [], all_vars, source=iris.domain)
+        self.assertTrue(sp.issparse(new_iris.X))
+        self.assertTrue(sp.issparse(new_iris.metas))
+        self.assertEqual(new_iris.X.shape, (len(iris), 2))
+        self.assertEqual(new_iris.metas.shape, (len(iris), 2))
+
+        # move back
+        back_iris = data.Table.from_table(iris.domain, new_iris)
+        self.assertEqual(back_iris.domain, iris.domain)
+        self.assertTrue(sp.issparse(back_iris.X))
+        self.assertTrue(sp.issparse(back_iris.metas))
+        self.assertEqual(back_iris.X.shape, iris.X.shape)
+        self.assertEqual(back_iris.metas.shape, iris.metas.shape)
+
+    def test_from_table_sparse_move_all_to_empty_metas(self):
+        iris = data.Table("iris")
+        iris.X = sp.csr_matrix(iris.X)
+        new_domain = data.domain.Domain([], iris.domain.class_vars,
+                                        iris.domain.attributes, source=iris.domain)
         new_iris = data.Table.from_table(new_domain, iris)
-        self.assertEqual(len(new_iris.domain.metas), n_all)
-        self.assertEqual(new_iris.metas.shape[1], n_all)
+        
+        self.assertTrue(sp.issparse(new_iris.X))
+        self.assertTrue(sp.issparse(new_iris.metas))
+        self.assertEqual(new_iris.X.shape, (len(iris), 0))
+        self.assertEqual(new_iris.metas.shape, (len(iris), 4))
+
+        # move back
+        back_iris = data.Table.from_table(iris.domain, new_iris)
+        self.assertEqual(back_iris.domain, iris.domain)
+        self.assertTrue(sp.issparse(back_iris.X))
+        self.assertTrue(sp.issparse(back_iris.metas))
+        self.assertEqual(back_iris.X.shape, iris.X.shape)
+        self.assertEqual(back_iris.metas.shape, iris.metas.shape)
+
+    def test_from_table_sparse_move_to_nonempty_metas(self):
+        brown = data.Table("brown-selected")
+        brown.X = sp.csr_matrix(brown.X)
+        n_attr = len(brown.domain.attributes)
+        n_metas = len(brown.domain.metas)
+        new_domain = data.domain.Domain(brown.domain.attributes[:-10], brown.domain.class_vars,
+                                        brown.domain.attributes[-10:] + brown.domain.metas,
+                                        source=brown.domain)
+        new_brown = data.Table.from_table(new_domain, brown)
+
+        self.assertTrue(sp.issparse(new_brown.X))
+        self.assertFalse(sp.issparse(new_brown.metas))
+        self.assertEqual(new_brown.X.shape, (len(new_brown), n_attr-10))
+        self.assertEqual(new_brown.metas.shape, (len(new_brown), n_metas+10))
+
+        # move back
+        back_brown = data.Table.from_table(brown.domain, new_brown)
+        self.assertEqual(brown.domain, back_brown.domain)
+        self.assertTrue(sp.issparse(back_brown.X))
+        self.assertFalse(sp.issparse(back_brown.metas))
+        self.assertEqual(back_brown.X.shape, brown.X.shape)
+        self.assertEqual(back_brown.metas.shape, brown.metas.shape)
 
     def assert_table_with_filter_matches(
             self, new_table, old_table,
@@ -2098,6 +2150,412 @@ class TestRowInstance(unittest.TestCase):
         np.testing.assert_array_equal(table.X[:, 0], np.arange(len(table)))
 
 
+class TestTableTranspose(unittest.TestCase):
+    def test_transpose_no_class(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        data = Table(Domain(attrs), np.arange(8).reshape((4, 2)))
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose is original
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_discrete_class(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        domain = Domain(attrs, [DiscreteVariable("cls", values=["a", "b"])])
+        data = Table(domain, np.arange(8).reshape((4, 2)),
+                     np.array([1, 1, 0, 0]))
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"cls": "b"}
+        att[1].attributes = {"cls": "b"}
+        att[2].attributes = {"cls": "a"}
+        att[3].attributes = {"cls": "a"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose is original
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_continuous_class(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        domain = Domain(attrs, [ContinuousVariable("cls")])
+        data = Table(domain, np.arange(8).reshape((4, 2)), np.arange(4, 0, -1))
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"cls": "4.000"}
+        att[1].attributes = {"cls": "3.000"}
+        att[2].attributes = {"cls": "2.000"}
+        att[3].attributes = {"cls": "1.000"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_missing_class(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        domain = Domain(attrs, [ContinuousVariable("cls")])
+        data = Table(domain, np.arange(8).reshape((4, 2)),
+                     np.array([np.nan, 3, 2, 1]))
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[1].attributes = {"cls": "3.000"}
+        att[2].attributes = {"cls": "2.000"}
+        att[3].attributes = {"cls": "1.000"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_multiple_class(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        class_vars = [ContinuousVariable("cls1"), ContinuousVariable("cls2")]
+        domain = Domain(attrs, class_vars)
+        data = Table(domain, np.arange(8).reshape((4, 2)),
+                     np.arange(8).reshape((4, 2)))
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"cls1": "0.000", "cls2": "1.000"}
+        att[1].attributes = {"cls1": "2.000", "cls2": "3.000"}
+        att[2].attributes = {"cls1": "4.000", "cls2": "5.000"}
+        att[3].attributes = {"cls1": "6.000", "cls2": "7.000"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_metas(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        metas = [StringVariable("m1")]
+        domain = Domain(attrs, metas=metas)
+        X = np.arange(8).reshape((4, 2))
+        M = np.array(["aa", "bb", "cc", "dd"])[:, None]
+        data = Table(domain, X, metas=M)
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"m1": "aa"}
+        att[1].attributes = {"m1": "bb"}
+        att[2].attributes = {"m1": "cc"}
+        att[3].attributes = {"m1": "dd"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_discrete_metas(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        metas = [DiscreteVariable("m1", values=["aa", "bb"])]
+        domain = Domain(attrs, metas=metas)
+        X = np.arange(8).reshape((4, 2))
+        M = np.array([0, 1, 0, 1])[:, None]
+        data = Table(domain, X, metas=M)
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"m1": "aa"}
+        att[1].attributes = {"m1": "bb"}
+        att[2].attributes = {"m1": "aa"}
+        att[3].attributes = {"m1": "bb"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose is original
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_continuous_metas(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        metas = [ContinuousVariable("m1")]
+        domain = Domain(attrs, metas=metas)
+        X = np.arange(8).reshape((4, 2))
+        M = np.array([0, 1, 0, 1])[:, None]
+        data = Table(domain, X, metas=M)
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"m1": "0.000"}
+        att[1].attributes = {"m1": "1.000"}
+        att[2].attributes = {"m1": "0.000"}
+        att[3].attributes = {"m1": "1.000"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose is original
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_missing_metas(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        metas = [StringVariable("m1")]
+        domain = Domain(attrs, metas=metas)
+        X = np.arange(8).reshape((4, 2))
+        M = np.array(["aa", "bb", "", "dd"])[:, None]
+        data = Table(domain, X, metas=M)
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"m1": "aa"}
+        att[1].attributes = {"m1": "bb"}
+        att[3].attributes = {"m1": "dd"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_multiple_metas(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        metas = [StringVariable("m1"), StringVariable("m2")]
+        domain = Domain(attrs, metas=metas)
+        X = np.arange(8).reshape((4, 2))
+        M = np.array([["aa", "aaa"], ["bb", "bbb"],
+                      ["cc", "ccc"], ["dd", "ddd"]])
+        data = Table(domain, X, metas=M)
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"m1": "aa", "m2": "aaa"}
+        att[1].attributes = {"m1": "bb", "m2": "bbb"}
+        att[2].attributes = {"m1": "cc", "m2": "ccc"}
+        att[3].attributes = {"m1": "dd", "m2": "ddd"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_class_and_metas(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        metas = [StringVariable("m1"), StringVariable("m2")]
+        domain = Domain(attrs, [ContinuousVariable("cls")], metas)
+        M = np.array([["aa", "aaa"], ["bb", "bbb"],
+                      ["cc", "ccc"], ["dd", "ddd"]])
+        data = Table(domain, np.arange(8).reshape((4, 2)), np.arange(1, 5), M)
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"cls": "1.000", "m1": "aa", "m2": "aaa"}
+        att[1].attributes = {"cls": "2.000", "m1": "bb", "m2": "bbb"}
+        att[2].attributes = {"cls": "3.000", "m1": "cc", "m2": "ccc"}
+        att[3].attributes = {"cls": "4.000", "m1": "dd", "m2": "ddd"}
+        domain = Domain(att, metas=[StringVariable("Feature name")])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array(["c1", "c2"])[:, None])
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes, {})
+
+    def test_transpose_attributes_of_attributes(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        attrs[0].attributes = {"attr1": "a", "attr2": "aa"}
+        attrs[1].attributes = {"attr1": "b", "attr2": "bb"}
+        domain = Domain(attrs)
+        data = Table(domain, np.arange(8).reshape((4, 2)))
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        metas = [StringVariable("Feature name"), StringVariable("attr1"),
+                 StringVariable("attr2")]
+        domain = Domain(att, metas=metas)
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array([["c1", "a", "aa"], ["c2", "b", "bb"]]))
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes,
+                             {"attr1": "a", "attr2": "aa"})
+
+    def test_transpose_attributes_of_attributes_missings(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        attrs[0].attributes = {"attr1": "a", "attr2": "aa"}
+        attrs[1].attributes = {"attr1": "b"}
+        domain = Domain(attrs)
+        data = Table(domain, np.arange(8).reshape((4, 2)))
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        metas = [StringVariable("Feature name"), StringVariable("attr1"),
+                 StringVariable("attr2")]
+        domain = Domain(att, metas=metas)
+        result = Table(domain, np.arange(8).reshape((4, 2)).T,
+                       metas=np.array([["c1", "a", "aa"], ["c2", "b", ""]]))
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes,
+                             {"attr1": "a", "attr2": "aa"})
+
+    def test_transpose_class_metas_attributes(self):
+        attrs = [ContinuousVariable("c1"), ContinuousVariable("c2")]
+        attrs[0].attributes = {"attr1": "a1", "attr2": "aa1"}
+        attrs[1].attributes = {"attr1": "b1", "attr2": "bb1"}
+        metas = [StringVariable("m1"), StringVariable("m2")]
+        domain = Domain(attrs, [ContinuousVariable("cls")], metas)
+        M = np.array([["aa", "aaa"], ["bb", "bbb"],
+                      ["cc", "ccc"], ["dd", "ddd"]])
+        data = Table(domain, np.arange(8).reshape((4, 2)), np.arange(1, 5), M)
+
+        att = [ContinuousVariable("Feature 1"), ContinuousVariable("Feature 2"),
+               ContinuousVariable("Feature 3"), ContinuousVariable("Feature 4")]
+        att[0].attributes = {"cls": "1.000", "m1": "aa", "m2": "aaa"}
+        att[1].attributes = {"cls": "2.000", "m1": "bb", "m2": "bbb"}
+        att[2].attributes = {"cls": "3.000", "m1": "cc", "m2": "ccc"}
+        att[3].attributes = {"cls": "4.000", "m1": "dd", "m2": "ddd"}
+        metas = [StringVariable("Feature name"), StringVariable("attr1"),
+                 StringVariable("attr2")]
+        domain = Domain(att, metas=metas)
+        M = np.array([["c1", "a1", "aa1"], ["c2", "b1", "bb1"]])
+        result = Table(domain, np.arange(8).reshape((4, 2)).T, metas=M)
+
+        # transpose and compare
+        self._compare_tables(result, Table.transpose(data))
+
+        # transpose of transpose
+        t = Table.transpose(Table.transpose(data), "Feature name")
+        self._compare_tables(data, t)
+
+        # original should not change
+        self.assertDictEqual(data.domain.attributes[0].attributes,
+                             {"attr1": "a1", "attr2": "aa1"})
+
+    def test_transpose(self):
+        zoo = Table("zoo")
+        t1 = Table.transpose(zoo)
+        t2 = Table.transpose(t1, "Feature name")
+        t3 = Table.transpose(t2)
+        self._compare_tables(zoo, t2)
+        self._compare_tables(t1, t3)
+
+    def _compare_tables(self, table1, table2):
+        self.assertEqual(table1.n_rows, table2.n_rows)
+        np.testing.assert_array_equal(table1.X, table2.X)
+        np.testing.assert_array_equal(table1.Y, table2.Y)
+        np.testing.assert_array_equal(table1.metas, table2.metas)
+        np.testing.assert_array_equal(table1.W, table2.W)
+
+        self.assertEqual([(type(x), x.name, x.attributes)
+                          for x in table1.domain.attributes],
+                         [(type(x), x.name, x.attributes)
+                          for x in table2.domain.attributes])
+        self.assertEqual([(type(x), x.name, x.attributes)
+                          for x in table1.domain.class_vars],
+                         [(type(x), x.name, x.attributes)
+                          for x in table2.domain.class_vars])
+        self.assertEqual([(type(x), x.name, x.attributes)
+                          for x in table1.domain.metas],
+                         [(type(x), x.name, x.attributes)
+                          for x in table2.domain.metas])
+
+
 if __name__ == "__main__":
     unittest.main()
 
