@@ -1,10 +1,14 @@
 import os
+import logging
 from itertools import chain, count
 from warnings import catch_warnings
 
 import numpy as np
-from PyQt4 import QtGui, QtCore
-from PyQt4.QtGui import QSizePolicy as Policy
+from AnyQt.QtWidgets import \
+    QStyle, QComboBox, QMessageBox, QFileDialog, QGridLayout, QLabel, \
+    QLineEdit
+from AnyQt.QtWidgets import QSizePolicy as Policy
+from AnyQt.QtCore import Qt, QTimer, QSize
 
 from Orange.canvas.gui.utils import OSX_NSURL_toLocalFile
 from Orange.data import Domain, DiscreteVariable, StringVariable
@@ -23,8 +27,17 @@ from Orange.widgets.utils.filedialogs import RecentPathsWComboMixin
 from Orange.widgets.utils.filedialogs import RecentPath
 
 
+log = logging.getLogger(__name__)
+
+
 def add_origin(examples, filename):
-    """Adds attribute with file location to each variable"""
+    """
+    Adds attribute with file location to each string variable
+    Used for relative filenames stored in string variables (e.g. pictures)
+    TODO: we should consider a cleaner solution (special variable type, ...)
+    """
+    if not filename:
+        return
     vars = examples.domain.variables + examples.domain.metas
     strings = [var for var in vars if var.is_string]
     dir_name, basename = os.path.split(filename)
@@ -40,7 +53,7 @@ class NamedURLModel(PyListModel):
 
     def data(self, index, role):
         data = super().data(index, role)
-        if role == QtCore.Qt.DisplayRole:
+        if role == Qt.DisplayRole:
             return self.mapping.get(data, data)
         return data
 
@@ -65,11 +78,11 @@ class XlsContextHandler(ContextHandler):
         return ContextHandler.NO_MATCH
 
 
-class LineEditSelectOnFocus(QtGui.QLineEdit):
+class LineEditSelectOnFocus(QLineEdit):
     def focusInEvent(self, event):
         super().focusInEvent(event)
         # If selectAll is called directly, placing the cursor unselects the text
-        QtCore.QTimer.singleShot(0, self.selectAll)
+        QTimer.singleShot(0, self.selectAll)
 
 
 class OWFile(widget.OWWidget, RecentPathsWComboMixin):
@@ -88,7 +101,7 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
     want_main_area = False
 
     SEARCH_PATHS = [("sample-datasets", get_sample_datasets_dir())]
-
+    SIZE_LIMIT = 1e7
     LOCAL_FILE, URL = range(2)
 
     settingsHandler = PerfectDomainContextHandler()
@@ -115,6 +128,10 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
                   for f in sorted(set(FileFormat.readers.values()),
                                   key=list(FileFormat.readers.values()).index)))
 
+    class Warning(widget.OWWidget.Warning):
+        file_too_big = widget.Msg("The file is too large to load automatically."
+                                  " Press Reload to load.")
+
     def __init__(self):
         super().__init__()
         RecentPathsWComboMixin.__init__(self)
@@ -123,13 +140,13 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
         self.loaded_file = ""
         self.reader = None
 
-        layout = QtGui.QGridLayout()
+        layout = QGridLayout()
         gui.widgetBox(self.controlArea, margin=0, orientation=layout)
         vbox = gui.radioButtons(None, self, "source", box=True, addSpace=True,
                                 callback=self.load_data, addToLayout=False)
 
         rb_button = gui.appendRadioButton(vbox, "File:", addToLayout=False)
-        layout.addWidget(rb_button, 0, 0, QtCore.Qt.AlignVCenter)
+        layout.addWidget(rb_button, 0, 0, Qt.AlignVCenter)
 
         box = gui.hBox(None, addToLayout=False, margin=0)
         box.setSizePolicy(Policy.MinimumExpanding, Policy.Fixed)
@@ -140,15 +157,14 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
 
         file_button = gui.button(
             None, self, '...', callback=self.browse_file, autoDefault=False)
-        file_button.setIcon(self.style().standardIcon(
-            QtGui.QStyle.SP_DirOpenIcon))
+        file_button.setIcon(self.style().standardIcon(QStyle.SP_DirOpenIcon))
         file_button.setSizePolicy(Policy.Maximum, Policy.Fixed)
         layout.addWidget(file_button, 0, 2)
 
         reload_button = gui.button(
             None, self, "Reload", callback=self.load_data, autoDefault=False)
         reload_button.setIcon(self.style().standardIcon(
-            QtGui.QStyle.SP_BrowserReload))
+            QStyle.SP_BrowserReload))
         reload_button.setSizePolicy(Policy.Fixed, Policy.Fixed)
         layout.addWidget(reload_button, 0, 3)
 
@@ -158,21 +174,21 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
                                         sendSelectedValue=True)
         self.sheet_combo.setSizePolicy(
             Policy.MinimumExpanding, Policy.Fixed)
-        self.sheet_label = QtGui.QLabel()
+        self.sheet_label = QLabel()
         self.sheet_label.setText('Sheet')
         self.sheet_label.setSizePolicy(
             Policy.MinimumExpanding, Policy.Fixed)
         self.sheet_box.layout().addWidget(
-            self.sheet_label, QtCore.Qt.AlignLeft)
+            self.sheet_label, Qt.AlignLeft)
         self.sheet_box.layout().addWidget(
-            self.sheet_combo, QtCore.Qt.AlignVCenter)
+            self.sheet_combo, Qt.AlignVCenter)
         layout.addWidget(self.sheet_box, 2, 1)
         self.sheet_box.hide()
 
         rb_button = gui.appendRadioButton(vbox, "URL:", addToLayout=False)
-        layout.addWidget(rb_button, 3, 0, QtCore.Qt.AlignVCenter)
+        layout.addWidget(rb_button, 3, 0, Qt.AlignVCenter)
 
-        self.url_combo = url_combo = QtGui.QComboBox()
+        self.url_combo = url_combo = QComboBox()
         url_model = NamedURLModel(self.sheet_names)
         url_model.wrap(self.recent_urls)
         url_combo.setLineEdit(LineEditSelectOnFocus())
@@ -205,19 +221,26 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
 
         self.apply_button = gui.button(
             box, self, "Apply", callback=self.apply_domain_edit)
-        self.apply_button.hide()
+        self.apply_button.setEnabled(False)
         self.apply_button.setFixedWidth(170)
-        self.editor_model.dataChanged.connect(self.apply_button.show)
+        self.editor_model.dataChanged.connect(
+            lambda: self.apply_button.setEnabled(True))
 
         self.set_file_list()
         # Must not call open_file from within __init__. open_file
         # explicitly re-enters the event loop (by a progress bar)
-        QtCore.QTimer.singleShot(0, self.load_data)
 
         self.setAcceptDrops(True)
 
+        if self.source == self.LOCAL_FILE and \
+                        os.path.getsize(self.last_path()) > self.SIZE_LIMIT:
+            self.Warning.file_too_big()
+            return
+
+        QTimer.singleShot(0, self.load_data)
+
     def sizeHint(self):
-        return QtCore.QSize(600, 550)
+        return QSize(600, 550)
 
     def select_file(self, n):
         assert n < len(self.recent_paths)
@@ -239,14 +262,14 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
         if in_demos:
             start_file = get_sample_datasets_dir()
             if not os.path.exists(start_file):
-                QtGui.QMessageBox.information(
+                QMessageBox.information(
                     None, "File",
                     "Cannot find the directory with documentation data sets")
                 return
         else:
             start_file = self.last_path() or os.path.expanduser("~/")
 
-        filename = QtGui.QFileDialog.getOpenFileName(
+        filename, _ = QFileDialog.getOpenFileName(
             self, 'Open Orange Data File', start_file, self.dlg_formats)
         if not filename:
             return
@@ -261,6 +284,8 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
         # file readers
         # pylint: disable=broad-except
         self.editor_model.set_domain(None)
+        self.apply_button.setEnabled(False)
+        self.Warning.file_too_big.clear()
 
         error = None
         try:
@@ -280,6 +305,7 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
                 try:
                     data = self.reader.read()
                 except Exception as ex:
+                    log.exception(ex)
                     error = ex
                 self.warning(warnings[-1].message.args[0] if warnings else '')
 
@@ -339,21 +365,33 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
 
     def _describe(self, table):
         domain = table.domain
-        text = "{} instance(s), {} feature(s), {} meta attribute(s)".format(
-            len(table), len(domain.attributes), len(domain.metas))
+        text = ""
+
+        attrs = getattr(table, "attributes", {})
+        descs = [attrs[desc]
+                 for desc in ("Name", "Description") if desc in attrs]
+        if len(descs) == 2:
+            descs[0] = "<b>{}</b>".format(descs[0])
+        if descs:
+            text += "<p>{}</p>".format("<br/>".join(descs))
+
+        text += "<p>{} instance(s), {} feature(s), {} meta attribute(s)".\
+            format(len(table), len(domain.attributes), len(domain.metas))
         if domain.has_continuous_class:
-            text += "\nRegression; numerical class."
+            text += "<br/>Regression; numerical class."
         elif domain.has_discrete_class:
-            text += "\nClassification; discrete class with {} values.".format(
-                len(domain.class_var.values))
+            text += "<br/>Classification; discrete class with {} values.".\
+                format(len(domain.class_var.values))
         elif table.domain.class_vars:
-            text += "\nMulti-target; {} target variables.".format(
+            text += "<br/>Multi-target; {} target variables.".format(
                 len(table.domain.class_vars))
         else:
-            text += "\nData has no target variable."
+            text += "<br/>Data has no target variable."
+        text += "</p>"
+
         if 'Timestamp' in table.domain:
             # Google Forms uses this header to timestamp responses
-            text += '\n\nFirst entry: {}\nLast entry: {}'.format(
+            text += '<p>First entry: {}<br/>Last entry: {}</p>'.format(
                 table[0, 'Timestamp'], table[-1, 'Timestamp'])
         return text
 
@@ -409,7 +447,7 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
         m = np.array(m, dtype=dtpe).T if len(m) else None
         table = Table.from_numpy(domain, X, y, m, self.data.W)
         self.send("Data", table)
-        self.apply_button.hide()
+        self.apply_button.setEnabled(False)
 
     def get_widget_name_extension(self):
         _, name = os.path.split(self.loaded_file)
@@ -466,7 +504,8 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
 
 if __name__ == "__main__":
     import sys
-    a = QtGui.QApplication(sys.argv)
+    from AnyQt.QtWidgets import QApplication
+    a = QApplication(sys.argv)
     ow = OWFile()
     ow.show()
     a.exec_()
