@@ -1,8 +1,15 @@
 from collections import OrderedDict
 from itertools import chain
 
-from PyQt4 import QtGui, QtCore
-from PyQt4.QtCore import Qt
+from AnyQt.QtWidgets import (
+    QWidget, QTableWidget, QHeaderView, QComboBox, QLineEdit, QToolButton,
+    QMessageBox, QMenu, QListView, QGridLayout, QPushButton, QSizePolicy
+)
+from AnyQt.QtGui import (
+    QDoubleValidator, QRegExpValidator, QStandardItemModel, QStandardItem,
+    QFontMetrics, QPalette
+)
+from AnyQt.QtCore import Qt, QPoint, QRegExp, QPersistentModelIndex
 
 from Orange.data import (ContinuousVariable, DiscreteVariable, StringVariable,
                          Table, TimeVariable)
@@ -70,17 +77,17 @@ class OWSelectRows(widget.OWWidget):
         self.data_desc = self.match_desc = self.nonmatch_desc = None
 
         box = gui.vBox(self.controlArea, 'Conditions', stretch=100)
-        self.cond_list = QtGui.QTableWidget(
-            box, showGrid=False, selectionMode=QtGui.QTableWidget.NoSelection)
+        self.cond_list = QTableWidget(
+            box, showGrid=False, selectionMode=QTableWidget.NoSelection)
         box.layout().addWidget(self.cond_list)
-        self.cond_list.setColumnCount(3)
+        self.cond_list.setColumnCount(4)
         self.cond_list.setRowCount(0)
         self.cond_list.verticalHeader().hide()
         self.cond_list.horizontalHeader().hide()
-        self.cond_list.resizeColumnToContents(0)
-        self.cond_list.horizontalHeader().setResizeMode(
-            QtGui.QHeaderView.Stretch)
-        self.cond_list.viewport().setBackgroundRole(QtGui.QPalette.Window)
+        for i in range(3):
+            self.cond_list.horizontalHeader().setSectionResizeMode(i, QHeaderView.Stretch)
+        self.cond_list.horizontalHeader().resizeSection(3, 30)
+        self.cond_list.viewport().setBackgroundRole(QPalette.Window)
 
         box2 = gui.hBox(box)
         gui.rubber(box2)
@@ -92,7 +99,7 @@ class OWSelectRows(widget.OWWidget):
             box2, self, "Remove All", callback=self.remove_all)
         gui.rubber(box2)
 
-        boxes = gui.widgetBox(self.controlArea, orientation=QtGui.QGridLayout())
+        boxes = gui.widgetBox(self.controlArea, orientation=QGridLayout())
         layout = boxes.layout()
         layout.setColumnStretch(0, 1)
         layout.setColumnStretch(1, 1)
@@ -129,15 +136,22 @@ class OWSelectRows(widget.OWWidget):
         row = model.rowCount()
         model.insertRow(row)
 
-        attr_combo = QtGui.QComboBox(
+        attr_combo = QComboBox(
             minimumContentsLength=12,
-            sizeAdjustPolicy=QtGui.QComboBox.AdjustToMinimumContentsLengthWithIcon)
+            sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLengthWithIcon)
         attr_combo.row = row
         for var in filter_visible(chain(self.data.domain.variables, self.data.domain.metas)):
             attr_combo.addItem(*gui.attributeItem(var))
         attr_combo.setCurrentIndex(attr or 0)
         self.cond_list.setCellWidget(row, 0, attr_combo)
 
+        index = QPersistentModelIndex(model.index(row, 3))
+        temp_button = QPushButton('×', self, flat=True,
+                                  styleSheet='* {font-size: 16pt; color: silver}'
+                                             '*:hover {color: black}')
+        temp_button.clicked.connect(lambda: self.remove_one(index.row()))
+        self.cond_list.setCellWidget(row, 3, temp_button)
+
         self.remove_all_button.setDisabled(False)
         self.set_new_operators(attr_combo, attr is not None,
                                condition_type, condition_value)
@@ -148,7 +162,7 @@ class OWSelectRows(widget.OWWidget):
 
     def add_all(self):
         if self.cond_list.rowCount():
-            Mb = QtGui.QMessageBox
+            Mb = QMessageBox
             if Mb.question(
                     self, "Remove existing filters",
                     "This will replace the existing filters with "
@@ -159,10 +173,19 @@ class OWSelectRows(widget.OWWidget):
         for i in range(len(domain.variables) + len(domain.metas)):
             self.add_row(i)
 
+    def remove_one(self, rownum):
+        self.remove_one_row(rownum)
+        self.conditions_changed()
+
     def remove_all(self):
         self.remove_all_rows()
         self.conditions_changed()
 
+    def remove_one_row(self, rownum):
+        self.cond_list.removeRow(rownum)
+        if self.cond_list.model().rowCount() == 0:
+            self.remove_all_button.setDisabled(True)
+
     def remove_all_rows(self):
         self.cond_list.clear()
         self.cond_list.setRowCount(0)
@@ -170,7 +193,7 @@ class OWSelectRows(widget.OWWidget):
 
     def set_new_operators(self, attr_combo, adding_all,
                           selected_index=None, selected_values=None):
-        oper_combo = QtGui.QComboBox()
+        oper_combo = QComboBox()
         oper_combo.row = attr_combo.row
         oper_combo.attr_combo = attr_combo
         var = self.data.domain[attr_combo.currentText()]
@@ -184,18 +207,18 @@ class OWSelectRows(widget.OWWidget):
     @staticmethod
     def _get_lineedit_contents(box):
         return [child.text() for child in getattr(box, "controls", [box])
-                if isinstance(child, QtGui.QLineEdit)]
+                if isinstance(child, QLineEdit)]
 
     @staticmethod
     def _get_value_contents(box):
         cont = []
         names = []
         for child in getattr(box, "controls", [box]):
-            if isinstance(child, QtGui.QLineEdit):
+            if isinstance(child, QLineEdit):
                 cont.append(child.text())
-            elif isinstance(child, QtGui.QComboBox):
+            elif isinstance(child, QComboBox):
                 cont.append(child.currentIndex())
-            elif isinstance(child, QtGui.QToolButton):
+            elif isinstance(child, QToolButton):
                 if child.popup is not None:
                     model = child.popup.list_view.model()
                     for row in range(model.rowCount()):
@@ -211,10 +234,10 @@ class OWSelectRows(widget.OWWidget):
                 raise TypeError('Type %s not supported.' % type(child))
         return tuple(cont)
 
-    class QDoubleValidatorEmpty(QtGui.QDoubleValidator):
+    class QDoubleValidatorEmpty(QDoubleValidator):
         def validate(self, input_, pos):
             if not input_:
-                return (QtGui.QDoubleValidator.Acceptable, input_, pos)
+                return (QDoubleValidator.Acceptable, input_, pos)
             else:
                 return super().validate(input_, pos)
 
@@ -225,10 +248,12 @@ class OWSelectRows(widget.OWWidget):
         #         child.setParent(None)
 
         def add_textual(contents):
-            le = gui.lineEdit(box, self, None)
+            le = gui.lineEdit(box, self, None,
+                              sizePolicy=QSizePolicy(QSizePolicy.Expanding,
+                                                     QSizePolicy.Expanding))
             if contents:
                 le.setText(contents)
-            le.setAlignment(QtCore.Qt.AlignRight)
+            le.setAlignment(Qt.AlignRight)
             le.editingFinished.connect(self.conditions_changed)
             return le
 
@@ -239,7 +264,7 @@ class OWSelectRows(widget.OWWidget):
 
         def add_datetime(contents):
             le = add_textual(contents)
-            le.setValidator(QtGui.QRegExpValidator(QtCore.QRegExp(TimeVariable.REGEX)))
+            le.setValidator(QRegExpValidator(QRegExp(TimeVariable.REGEX)))
             return le
 
         var = self.data.domain[oper_combo.attr_combo.currentText()]
@@ -263,7 +288,7 @@ class OWSelectRows(widget.OWWidget):
                 button.var_type = vartype(var)
                 self.cond_list.setCellWidget(oper_combo.row, 2, button)
             else:
-                combo = QtGui.QComboBox()
+                combo = QComboBox()
                 combo.addItems([""] + var.values)
                 if lc[0]:
                     combo.setCurrentIndex(int(lc[0]))
@@ -282,7 +307,6 @@ class OWSelectRows(widget.OWWidget):
                 if oper > 5:
                     gui.widgetLabel(box, " and ")
                     box.controls.append(validator(lc[1]))
-                gui.rubber(box)
             elif var.is_string:
                 box.controls = [add_textual(lc[0])]
                 if oper in [6, 7]:
@@ -319,7 +343,7 @@ class OWSelectRows(widget.OWWidget):
         self.update_info(data, self.data_in_variables, "In: ")
         for attr, cond_type, cond_value in self.conditions:
             attrs = [a.name for a in
-                     data.domain.variables + data.domain.metas]
+                     filter_visible(chain(data.domain.variables, data.domain.metas))]
             if attr in attrs:
                 self.add_row(attrs.index(attr), cond_type, cond_value)
         self.unconditional_commit()
@@ -344,7 +368,7 @@ class OWSelectRows(widget.OWWidget):
     def commit(self):
         matching_output = self.data
         non_matching_output = None
-        self.error(21)
+        self.error()
         if self.data:
             domain = self.data.domain
             conditions = []
@@ -352,17 +376,18 @@ class OWSelectRows(widget.OWWidget):
                 attr_index = domain.index(attr_name)
                 attr = domain[attr_index]
 
-                # Parse datetime strings into floats
-                if isinstance(attr, TimeVariable):
-                    try:
-                        values = [attr.parse(v) for v in values]
-                    except ValueError as e:
-                        self.error(21, e.args[0])
-                        return
-
                 if attr.is_continuous:
                     if any(not v for v in values):
                         continue
+
+                    # Parse datetime strings into floats
+                    if isinstance(attr, TimeVariable):
+                        try:
+                            values = [attr.parse(v) for v in values]
+                        except ValueError as e:
+                            self.error(e.args[0])
+                            return
+
                     filter = data_filter.FilterContinuous(
                         attr_index, oper, *[float(v) for v in values])
                 elif attr.is_string:
@@ -503,30 +528,30 @@ class OWSelectRows(widget.OWWidget):
                   nonmatch_inst > 0 and "{} instances".format(nonmatch_inst))))
 
 
-class CheckBoxPopup(QtGui.QWidget):
+class CheckBoxPopup(QWidget):
     def __init__(self, var, lc, widget_parent=None, widget=None):
-        QtGui.QWidget.__init__(self)
+        QWidget.__init__(self)
 
-        self.list_view = QtGui.QListView()
+        self.list_view = QListView()
         text = []
-        model = QtGui.QStandardItemModel(self.list_view)
+        model = QStandardItemModel(self.list_view)
         for (i, val) in enumerate(var.values):
-            item = QtGui.QStandardItem(val)
+            item = QStandardItem(val)
             item.setCheckable(True)
             if i + 1 in lc:
-                item.setCheckState(QtCore.Qt.Checked)
+                item.setCheckState(Qt.Checked)
                 text.append(val)
             model.appendRow(item)
         model.itemChanged.connect(widget_parent.conditions_changed)
         self.list_view.setModel(model)
 
-        layout = QtGui.QGridLayout(self)
+        layout = QGridLayout(self)
         layout.addWidget(self.list_view)
         layout.setContentsMargins(0, 0, 0, 0)
         self.setLayout(layout)
 
         self.adjustSize()
-        self.setWindowFlags(QtCore.Qt.Popup)
+        self.setWindowFlags(Qt.Popup)
 
         self.widget = widget
         self.widget.desc_text = ', '.join(text)
@@ -535,15 +560,15 @@ class CheckBoxPopup(QtGui.QWidget):
     def moved(self):
         point = self.widget.rect().bottomRight()
         global_point = self.widget.mapToGlobal(point)
-        self.move(global_point - QtCore.QPoint(self.width(), 0))
+        self.move(global_point - QPoint(self.width(), 0))
 
 
-class DropDownToolButton(QtGui.QToolButton):
+class DropDownToolButton(QToolButton):
     def __init__(self, parent, var, lc):
-        QtGui.QToolButton.__init__(self, parent)
+        QToolButton.__init__(self, parent)
         self.desc_text = ''
         self.popup = CheckBoxPopup(var, lc, parent, self)
-        self.setMenu(QtGui.QMenu()) # to show arrow
+        self.setMenu(QMenu()) # to show arrow
         self.clicked.connect(self.open_popup)
 
     def open_popup(self):
@@ -551,9 +576,8 @@ class DropDownToolButton(QtGui.QToolButton):
         self.popup.show()
 
     def set_text(self):
-        metrics = QtGui.QFontMetrics(self.font())
-        self.setText(metrics.elidedText(self.desc_text,
-                                        QtCore.Qt.ElideRight,
+        metrics = QFontMetrics(self.font())
+        self.setText(metrics.elidedText(self.desc_text, Qt.ElideRight,
                                         self.width() - 15))
 
     def resizeEvent(self, QResizeEvent):
@@ -561,7 +585,8 @@ class DropDownToolButton(QtGui.QToolButton):
 
 
 def test():
-    app = QtGui.QApplication([])
+    from AnyQt.QtWidgets import QApplication
+    app = QApplication([])
     w = OWSelectRows()
     w.set_data(Table("zoo"))
     w.show()
