@@ -3,17 +3,23 @@
 from math import isnan, isinf
 import unicodedata
 
-from PyQt4.QtGui import (
-    QGridLayout, QTableView, QStandardItemModel, QStandardItem,
-    QItemSelectionModel, QItemSelection, QFont, QHeaderView, QBrush, QColor,
-    QStyledItemDelegate)
-from PyQt4.QtCore import Qt, QSize
-
+from AnyQt.QtWidgets import (
+    QGridLayout, QTableView,QHeaderView, QStyledItemDelegate
+)
+from AnyQt.QtGui import (
+    QFont, QBrush, QColor, QStandardItemModel, QStandardItem
+)
+from AnyQt.QtCore import (
+    Qt, QSize, QItemSelectionModel, QItemSelection, QT_VERSION
+)
 import numpy
 import sklearn.metrics as skl_metrics
 
 import Orange
 from Orange.widgets import widget, settings, gui
+from Orange.widgets.utils.annotated_data import (create_annotated_table,
+                                                 ANNOTATED_DATA_SIGNAL_NAME)
+from Orange.widgets.widget import Msg
 
 
 def confusion_matrix(res, index):
@@ -78,12 +84,14 @@ class OWConfusionMatrix(widget.OWWidget):
     priority = 1001
 
     inputs = [("Evaluation Results", Orange.evaluation.Results, "set_results")]
-    outputs = [("Selected Data", Orange.data.Table)]
+    outputs = [("Selected Data", Orange.data.Table, widget.Default),
+               (ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)]
 
     quantities = ["Number of instances",
                   "Proportion of predicted",
                   "Proportion of actual"]
 
+    settings_version = 1
     settingsHandler = settings.ClassValuesContextHandler()
 
     selected_learner = settings.Setting([0], schema_only=True)
@@ -99,6 +107,9 @@ class OWConfusionMatrix(widget.OWWidget):
             "data instances",
             "click_cell")]
 
+    class Error(widget.OWWidget.Error):
+        no_regression = Msg("Confusion Matrix cannot show regression results.")
+
     def __init__(self):
         super().__init__()
 
@@ -203,7 +214,7 @@ class OWConfusionMatrix(widget.OWWidget):
 
         hor_header = self.tableview.horizontalHeader()
         if len(' '.join(self.headers)) < 120:
-            hor_header.setResizeMode(QHeaderView.ResizeToContents)
+            hor_header.setSectionResizeMode(QHeaderView.ResizeToContents)
         else:
             hor_header.setDefaultSectionSize(60)
         self.tablemodel.setRowCount(nclasses + 3)
@@ -219,10 +230,13 @@ class OWConfusionMatrix(widget.OWWidget):
 
         data = None
         if results is not None and results.data is not None:
-            data = results.data
+            data = results.data[results.row_indices]
 
         if data is not None and not data.domain.has_discrete_class:
-            self.warning("Confusion Matrix cannot show regression results.")
+            self.Error.no_regression()
+            data = results = None
+        else:
+            self.Error.no_regression.clear()
 
         self.results = results
         self.data = data
@@ -251,7 +265,8 @@ class OWConfusionMatrix(widget.OWWidget):
             self._init_table(len(class_values))
             self.openContext(data.domain.class_var)
             if not prev_sel_learner or prev_sel_learner[0] >= len(self.learners):
-                self.selected_learner[:] = [0]
+                if self.learners:
+                    self.selected_learner[:] = [0]
             else:
                 self.selected_learner[:] = prev_sel_learner
             self._update()
@@ -324,14 +339,12 @@ class OWConfusionMatrix(widget.OWWidget):
             predicted = self.results.predicted[self.selected_learner[0]]
             selected = [i for i, t in enumerate(zip(actual, predicted))
                         if t in indices]
-            row_indices = self.results.row_indices[selected]
 
             extra = []
             class_var = self.data.domain.class_var
             metas = self.data.domain.metas
 
             if self.append_predictions:
-                predicted = numpy.array(predicted[selected], dtype=object)
                 extra.append(predicted.reshape(-1, 1))
                 var = Orange.data.DiscreteVariable(
                     "{}({})".format(class_var.name, learner_name),
@@ -341,17 +354,16 @@ class OWConfusionMatrix(widget.OWWidget):
 
             if self.append_probabilities and \
                     self.results.probabilities is not None:
-                probs = self.results.probabilities[self.selected_learner[0],
-                                                   selected]
+                probs = self.results.probabilities[self.selected_learner[0]]
                 extra.append(numpy.array(probs, dtype=object))
                 pvars = [Orange.data.ContinuousVariable("p({})".format(value))
                          for value in class_var.values]
                 metas = metas + tuple(pvars)
 
-            X = self.data.X[row_indices]
-            Y = self.data.Y[row_indices]
-            M = self.data.metas[row_indices]
-            row_ids = self.data.ids[row_indices]
+            X = self.data.X
+            Y = self.data.Y
+            M = self.data.metas
+            row_ids = self.data.ids
 
             M = numpy.hstack((M,) + tuple(extra))
             domain = Orange.data.Domain(
@@ -363,10 +375,19 @@ class OWConfusionMatrix(widget.OWWidget):
             data.ids = row_ids
             data.name = learner_name
 
+            if selected:
+                annotated_data = create_annotated_table(data, selected)
+                data = data[selected]
+            else:
+                annotated_data = create_annotated_table(data, [])
+                data = None
+
         else:
             data = None
+            annotated_data = None
 
         self.send("Selected Data", data)
+        self.send(ANNOTATED_DATA_SIGNAL_NAME, annotated_data)
 
     def _invalidate(self):
         indices = self.tableview.selectedIndexes()
@@ -463,8 +484,21 @@ class OWConfusionMatrix(widget.OWWidget):
                        self.quantities[self.selected_quantity].lower()),
                 self.tableview)
 
+    @classmethod
+    def migrate_settings(cls, settings, version):
+        super().migrate_settings(settings, version)
+        if not version:
+            # For some period of time the 'selected_learner' property was
+            # changed from List[int] -> int
+            # (commit 4e49bb3fd0e11262f3ebf4b1116a91a4b49cc982) and then back
+            # again (commit 8a492d79a2e17154a0881e24a05843406c8892c0)
+            if "selected_learner" in settings and \
+                    isinstance(settings["selected_learner"], int):
+                settings["selected_learner"] = [settings["selected_learner"]]
+
+
 if __name__ == "__main__":
-    from PyQt4.QtGui import QApplication
+    from AnyQt.QtWidgets import QApplication
 
     APP = QApplication([])
     w = OWConfusionMatrix()
