@@ -1,6 +1,6 @@
 import bottleneck as bn
 import numpy
-from PyQt4.QtCore import Qt
+from AnyQt.QtCore import Qt
 from scipy.sparse import issparse
 
 import Orange.data
@@ -42,6 +42,7 @@ class OWDistances(OWWidget):
         no_continuous_features = Msg("No continuous features")
         dense_metric_sparse_data = Msg("Selected metric does not support sparse data")
         empty_data = Msg("Empty data (shape = {})")
+        too_few_observations = Msg("Too few observations for the number of dimensions")
 
     class Warning(OWWidget.Warning):
         ignoring_discrete = Msg("Ignoring discrete features")
@@ -99,11 +100,11 @@ class OWDistances(OWWidget):
         self.send("Distances", self.compute_distances(metric, self.data))
 
     def compute_distances(self, metric, data):
+        self.clear_messages()
+
         if data is None:
             return
 
-        self.clear_messages()
-
         if issparse(data.X) and not metric.supports_sparse:
             self.Error.dense_metric_sparse_data()
             return
@@ -129,6 +130,14 @@ class OWDistances(OWWidget):
             self.Error.empty_data(data.X.shape)
             return
 
+        if isinstance(metric, distance.MahalanobisDistance):
+            n, m = data.X.shape
+            if self.axis == 1:
+                n, m = m, n
+            if n <= m:
+                self.Error.too_few_observations()
+                return
+
         if isinstance(metric, distance.MahalanobisDistance):
             # Mahalanobis distance has to be trained before it can be used
             # to compute distances
