@@ -9,13 +9,19 @@ import pkg_resources
 import numpy
 import scipy.spatial.distance
 
-from PyQt4 import QtGui
-from PyQt4.QtCore import Qt, QEvent
+from AnyQt.QtWidgets import (
+    QFormLayout, QHBoxLayout, QGroupBox, QToolButton, QActionGroup, QAction, QApplication,
+    QGraphicsLineItem
+)
+from AnyQt.QtGui import QColor, QPen, QBrush, QPainter, QKeySequence, QCursor, QIcon
+from AnyQt.QtCore import Qt, QEvent
+
 import pyqtgraph as pg
 import pyqtgraph.graphicsItems.ScatterPlotItem
 
 import Orange.data
 import Orange.projection
+from Orange.projection.manifold import torgerson
 import Orange.distance
 from Orange.data.domain import filter_visible
 import Orange.misc
@@ -23,40 +29,9 @@ from Orange.widgets import widget, gui, settings
 from Orange.widgets.utils import colorpalette, itemmodels
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.canvas import report
-
-
-def torgerson(distances, n_components=2):
-    """
-    Perform classical mds (Torgerson scaling).
-
-    ..note ::
-        If the distances are euclidean then this is equivalent to projecting
-        the original data points to the first `n` principal components.
-
-    """
-    distances = numpy.asarray(distances)
-    assert distances.shape[0] == distances.shape[1]
-    N = distances.shape[0]
-    # O ^ 2
-    D_sq = distances ** 2
-
-    # double center the D_sq
-    rsum = numpy.sum(D_sq, axis=1, keepdims=True)
-    csum = numpy.sum(D_sq, axis=0, keepdims=True)
-    total = numpy.sum(csum)
-    D_sq -= rsum / N
-    D_sq -= csum / N
-    D_sq += total / (N ** 2)
-    B = numpy.multiply(D_sq, -0.5, out=D_sq)
-
-    U, L, _ = numpy.linalg.svd(B)
-    if n_components > N:
-        U = numpy.hstack((U, numpy.zeros((N, n_components - N))))
-        L = numpy.hstack((L, numpy.zeros((n_components - N))))
-    U = U[:, :n_components]
-    L = L[:n_components]
-    D = numpy.diag(numpy.sqrt(L))
-    return numpy.dot(U, D)
+from Orange.widgets.widget import Msg, OWWidget
+from Orange.widgets.utils.annotated_data import (create_annotated_table,
+                                                 ANNOTATED_DATA_SIGNAL_NAME)
 
 
 def stress(X, D):
@@ -69,7 +44,7 @@ def stress(X, D):
 
 
 def make_pen(color, width=1.5, style=Qt.SolidLine, cosmetic=False):
-    pen = QtGui.QPen(color, width, style)
+    pen = QPen(color, width, style)
     pen.setCosmetic(cosmetic)
     return pen
 
@@ -82,15 +57,15 @@ class ScatterPlotItem(pg.ScatterPlotItem):
 
     def paint(self, painter, option, widget=None):
         if self.opts["pxMode"]:
-            painter.setRenderHint(QtGui.QPainter.SmoothPixmapTransform, True)
+            painter.setRenderHint(QPainter.SmoothPixmapTransform, True)
 
         if self.opts["antialias"]:
-            painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
+            painter.setRenderHint(QPainter.Antialiasing, True)
 
         super().paint(painter, option, widget)
 
 
-class OWMDS(widget.OWWidget):
+class OWMDS(OWWidget):
     name = "MDS"
     description = "Two-dimensional data projection by multidimensional " \
                   "scaling constructed from a distance matrix."
@@ -100,7 +75,7 @@ class OWMDS(widget.OWWidget):
               ("Data Subset", Orange.data.Table, "set_subset_data")]
 
     outputs = [("Selected Data", Orange.data.Table, widget.Default),
-               ("Data", Orange.data.Table)]
+               (ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)]
 
     #: Initialization type
     PCA, Random = 0, 1
@@ -152,12 +127,26 @@ class OWMDS(widget.OWWidget):
 
     graph_name = "plot.plotItem"
 
+    class Error(OWWidget.Error):
+        not_enough_rows = Msg("Input data needs at least 2 rows")
+        matrix_too_small = Msg("Input matrix must be at least 2x2")
+        no_attributes = Msg("Data has no attributes")
+        mismatching_dimensions = \
+            Msg("Data and distances dimensions do not match.")
+
     def __init__(self):
         super().__init__()
-        self.matrix = None
-        self.data = None
+        #: Input dissimilarity matrix
+        self.matrix = None  # type: Optional[Orange.misc.DistMatrix]
+        #: Effective data used for plot styling/annotations. Can be from the
+        #: input signal (`self.signal_data`) or the input matrix
+        #: (`self.matrix.data`)
+        self.data = None  # type: Optional[Orange.data.Table]
+        #: Input subset data table
         self.subset_data = None  # type: Optional[Orange.data.Table]
-        self.matrix_data = None
+        #: Data table from the `self.matrix.row_items` (if present)
+        self.matrix_data = None  # type: Optional[Orange.data.Table]
+        #: Input data table
         self.signal_data = None
 
         self._pen_data = None
@@ -179,10 +168,10 @@ class OWMDS(widget.OWWidget):
         self.__draw_similar_pairs = False
 
         box = gui.vBox(self.controlArea, "MDS Optimization")
-        form = QtGui.QFormLayout(
+        form = QFormLayout(
             labelAlignment=Qt.AlignLeft,
             formAlignment=Qt.AlignLeft,
-            fieldGrowthPolicy=QtGui.QFormLayout.AllNonFixedFieldsGrow,
+            fieldGrowthPolicy=QFormLayout.AllNonFixedFieldsGrow,
             verticalSpacing=10
         )
 
@@ -190,9 +179,9 @@ class OWMDS(widget.OWWidget):
                     gui.spin(box, self, "max_iter", 10, 10 ** 4, step=1))
 
         form.addRow("Initialization:",
-                    gui.comboBox(box, self, "initialization",
-                                 items=["PCA (Torgerson)", "Random"],
-                                 callback=self.__invalidate_embedding))
+                    gui.radioButtons(box, self, "initialization",
+                                     btnLabels=("PCA (Torgerson)", "Random"),
+                                     callback=self.__invalidate_embedding))
 
         box.layout().addLayout(form)
         form.addRow("Refresh:",
@@ -237,10 +226,10 @@ class OWMDS(widget.OWWidget):
             gui.indentedBox(box), self, 'label_only_selected',
             'Label only selected points', callback=self._on_label_index_changed)
 
-        form = QtGui.QFormLayout(
+        form = QFormLayout(
             labelAlignment=Qt.AlignLeft,
             formAlignment=Qt.AlignLeft,
-            fieldGrowthPolicy=QtGui.QFormLayout.AllNonFixedFieldsGrow,
+            fieldGrowthPolicy=QFormLayout.AllNonFixedFieldsGrow,
             verticalSpacing=10
         )
         form.addRow("Symbol size:",
@@ -269,30 +258,30 @@ class OWMDS(widget.OWWidget):
 
         gui.rubber(self.controlArea)
 
-        box = QtGui.QGroupBox("Zoom/Select", )
-        box.setLayout(QtGui.QHBoxLayout())
+        box = QGroupBox("Zoom/Select", )
+        box.setLayout(QHBoxLayout())
         box.layout().setContentsMargins(2, 2, 2, 2)
 
-        group = QtGui.QActionGroup(self, exclusive=True)
+        group = QActionGroup(self, exclusive=True)
 
         def icon(name):
             path = "icons/Dlg_{}.png".format(name)
             path = pkg_resources.resource_filename(widget.__name__, path)
-            return QtGui.QIcon(path)
+            return QIcon(path)
 
-        action_select = QtGui.QAction(
+        action_select = QAction(
             "Select", self, checkable=True, checked=True, icon=icon("arrow"),
-            shortcut=QtGui.QKeySequence(Qt.ControlModifier + Qt.Key_1))
-        action_zoom = QtGui.QAction(
+            shortcut=QKeySequence(Qt.ControlModifier + Qt.Key_1))
+        action_zoom = QAction(
             "Zoom", self, checkable=True, checked=False, icon=icon("zoom"),
-            shortcut=QtGui.QKeySequence(Qt.ControlModifier + Qt.Key_2))
-        action_pan = QtGui.QAction(
+            shortcut=QKeySequence(Qt.ControlModifier + Qt.Key_2))
+        action_pan = QAction(
             "Pan", self, checkable=True, checked=False, icon=icon("pan_hand"),
-            shortcut=QtGui.QKeySequence(Qt.ControlModifier + Qt.Key_3))
+            shortcut=QKeySequence(Qt.ControlModifier + Qt.Key_3))
 
-        action_reset_zoom = QtGui.QAction(
+        action_reset_zoom = QAction(
             "Zoom to fit", self, icon=icon("zoom_reset"),
-            shortcut=QtGui.QKeySequence(Qt.ControlModifier + Qt.Key_0))
+            shortcut=QKeySequence(Qt.ControlModifier + Qt.Key_0))
         action_reset_zoom.triggered.connect(
             lambda: self.plot.autoRange(padding=0.1,
                                         items=[self._scatter_item]))
@@ -303,7 +292,7 @@ class OWMDS(widget.OWWidget):
         action_select.setChecked(True)
 
         def button(action):
-            b = QtGui.QToolButton()
+            b = QToolButton()
             b.setToolButtonStyle(Qt.ToolButtonIconOnly)
             b.setDefaultAction(action)
             return b
@@ -334,7 +323,7 @@ class OWMDS(widget.OWWidget):
         self.plot.getPlotItem().hideAxis("bottom")
         self.plot.getPlotItem().hideAxis("left")
         self.plot.getPlotItem().hideButtons()
-        self.plot.setRenderHint(QtGui.QPainter.Antialiasing)
+        self.plot.setRenderHint(QPainter.Antialiasing)
         self.mainArea.layout().addWidget(self.plot)
 
         self.selection_tool = PlotSelectionTool(parent=self)
@@ -357,9 +346,9 @@ class OWMDS(widget.OWWidget):
                 active, cur = self.pan_tool, Qt.OpenHandCursor
             self.current_tool = active
             self.current_tool.setViewBox(self.plot.getViewBox())
-            self.plot.getViewBox().setCursor(QtGui.QCursor(cur))
+            self.plot.getViewBox().setCursor(QCursor(cur))
 
-        group.triggered[QtGui.QAction].connect(activate_tool)
+        group.triggered[QAction].connect(activate_tool)
 
         self._initialize()
 
@@ -371,6 +360,12 @@ class OWMDS(widget.OWWidget):
         ----------
         data : Optional[Orange.data.Table]
         """
+        if data is not None and len(data) < 2:
+            self.Error.not_enough_rows()
+            data = None
+        else:
+            self.Error.not_enough_rows.clear()
+
         self.signal_data = data
 
         if self.matrix is not None and data is not None and len(self.matrix) == len(data):
@@ -389,6 +384,13 @@ class OWMDS(widget.OWWidget):
         ----------
         matrix : Optional[Orange.misc.DistMatrix]
         """
+
+        if matrix is not None and len(matrix) < 2:
+            self.Error.matrix_too_small()
+            matrix = None
+        else:
+            self.Error.matrix_too_small.clear()
+
         self.matrix = matrix
         if matrix is not None and matrix.row_items:
             self.matrix_data = matrix.row_items
@@ -452,7 +454,6 @@ class OWMDS(widget.OWWidget):
             self.color_value = attr
             self.shape_value = attr
         else:
-            # initialize the graph state from data
             domain = self.data.domain
             all_vars = list(filter_visible(domain.variables + domain.metas))
             cd_vars = [var for var in all_vars if var.is_primitive()]
@@ -480,6 +481,7 @@ class OWMDS(widget.OWWidget):
         # clear everything
         self.closeContext()
         self._clear()
+        self.Error.clear()
         self.data = None
         self._effective_matrix = None
         self.embedding = None
@@ -488,25 +490,27 @@ class OWMDS(widget.OWWidget):
         if self.signal_data is None and self.matrix is None:
             return
 
-        if self.signal_data and self.matrix is not None and len(self.signal_data) != len(self.matrix):
-            self.error(1, "Data and distances dimensions do not match.")
+        if self.signal_data is not None and self.matrix is not None and \
+                len(self.signal_data) != len(self.matrix):
+            self.Error.mismatching_dimensions()
             self._update_plot()
             return
 
-        self.error(1)
-
-        if self.signal_data:
+        if self.signal_data is not None:
             self.data = self.signal_data
-        elif self.matrix_data:
+        elif self.matrix_data is not None:
             self.data = self.matrix_data
 
         if self.matrix is not None:
             self._effective_matrix = self.matrix
             if self.matrix.axis == 0 and self.data is self.matrix_data:
                 self.data = None
-        else:
+        elif self.data.domain.attributes:
             preprocessed_data = Orange.projection.MDS().preprocess(self.data)
             self._effective_matrix = Orange.distance.Euclidean(preprocessed_data)
+        else:
+            self.Error.no_attributes()
+            return
 
         self.update_controls()
         self.openContext(self.data)
@@ -535,13 +539,7 @@ class OWMDS(widget.OWWidget):
     def __start(self):
         self.__draw_similar_pairs = False
         X = self._effective_matrix
-
-        if self.embedding is not None:
-            init = self.embedding
-        elif self.initialization == OWMDS.PCA:
-            init = torgerson(X, n_components=2)
-        else:
-            init = None
+        init = self.embedding
 
         # number of iterations per single GUI update step
         _, step_size = OWMDS.RefreshRate[self.refresh_rate]
@@ -556,14 +554,16 @@ class OWMDS(widget.OWWidget):
             done = False
             iterations_done = 0
             oldstress = numpy.finfo(numpy.float).max
+            init_type = "PCA" if self.initialization == OWMDS.PCA else "random"
 
             while not done:
                 step_iter = min(max_iter - iterations_done, step)
                 mds = Orange.projection.MDS(
                     dissimilarity="precomputed", n_components=2,
-                    n_init=1, max_iter=step_iter)
+                    n_init=1, max_iter=step_iter,
+                    init_type=init_type, init_data=init)
 
-                mdsfit = mds.fit(X, init=init)
+                mdsfit = mds(X)
                 iterations_done += step_iter
 
                 embedding, stress = mdsfit.embedding_, mdsfit.stress_
@@ -610,7 +610,7 @@ class OWMDS(widget.OWWidget):
             self.setStatusMessage("Running")
             self.runbutton.setText("Stop")
             self.__state = OWMDS.Running
-            QtGui.QApplication.postEvent(self, QEvent(QEvent.User))
+            QApplication.postEvent(self, QEvent(QEvent.User))
         else:
             self.setStatusMessage("")
             self.runbutton.setText("Start")
@@ -636,7 +636,7 @@ class OWMDS(widget.OWWidget):
             self._update_plot()
             self.plot.autoRange(padding=0.1, items=[self._scatter_item])
             # schedule next update
-            QtGui.QApplication.postEvent(
+            QApplication.postEvent(
                 self, QEvent(QEvent.User), Qt.LowEventPriority)
 
     def customEvent(self, event):
@@ -653,7 +653,7 @@ class OWMDS(widget.OWWidget):
                     "A rogue `proccessEvents` is on the loose.",
                     RuntimeWarning)
                 # re-schedule the update iteration.
-                QtGui.QApplication.postEvent(self, QEvent(QEvent.User))
+                QApplication.postEvent(self, QEvent(QEvent.User))
         return super().customEvent(event)
 
     def __invalidate_embedding(self):
@@ -808,7 +808,7 @@ class OWMDS(widget.OWWidget):
                 pen_data = mdsplotutils.pen_data(color_data * 0.8, pointflags)
                 brush_data = mdsplotutils.brush_data(color_data)
             else:
-                pen_data = make_pen(QtGui.QColor(Qt.darkGray), cosmetic=True)
+                pen_data = make_pen(QColor(Qt.darkGray), cosmetic=True)
                 if self._selection_mask is not None:
                     pen_data = numpy.array(
                         [pen_data, plotstyle.selected_pen])
@@ -823,7 +823,7 @@ class OWMDS(widget.OWWidget):
             if self._subset_mask is not None and have_data and \
                     self._subset_mask.shape == (size, ):
                 # clear brush fill for non subset data
-                brush_data[~self._subset_mask] = QtGui.QBrush(Qt.NoBrush)
+                brush_data[~self._subset_mask] = QBrush(Qt.NoBrush)
 
             self._pen_data = pen_data
             self._brush_data = brush_data
@@ -913,13 +913,18 @@ class OWMDS(widget.OWWidget):
                 fpairs[::2] = indcs[0][sorted]
                 fpairs[1::2] = indcs[1][sorted]
             for i in range(int(len(emb_x[self._similar_pairs]) / 2)):
-                item = QtGui.QGraphicsLineItem(
+                item = QGraphicsLineItem(
                     emb_x[self._similar_pairs][i * 2],
                     emb_y[self._similar_pairs][i * 2],
                     emb_x[self._similar_pairs][i * 2 + 1],
                     emb_y[self._similar_pairs][i * 2 + 1]
                 )
-                pen = QtGui.QPen(QtGui.QBrush(QtGui.QColor(204, 204, 204)), 2)
+                if item.line().isNull():
+                    # Null (zero length) line causes bad rendering artifacts
+                    # in Qt when using the raster graphics system
+                    # (see gh-issue: 1668).
+                    continue
+                pen = QPen(QBrush(QColor(204, 204, 204)), 2)
                 pen.setCosmetic(True)
                 item.setPen(pen)
                 self.plot.addItem(item)
@@ -1011,13 +1016,14 @@ class OWMDS(widget.OWWidget):
             elif self.output_embedding_role == OWMDS.MetaRole:
                 output.metas[:, -2:] = embedding.X
 
-        self.send("Data", output)
         if output is not None and self._selection_mask is not None and \
                 numpy.any(self._selection_mask):
             subset = output[self._selection_mask]
         else:
             subset = None
         self.send("Selected Data", subset)
+        self.send(ANNOTATED_DATA_SIGNAL_NAME,
+                  create_annotated_table(output, self._selection_mask))
 
     def onDeleteWidget(self):
         super().onDeleteWidget()
@@ -1040,10 +1046,10 @@ class OWMDS(widget.OWWidget):
              if region.contains(spot.pos())],
             dtype=int)
 
-        if not QtGui.QApplication.keyboardModifiers():
+        if not QApplication.keyboardModifiers():
             self._selection_mask = None
 
-        self.select_indices(indices, QtGui.QApplication.keyboardModifiers())
+        self.select_indices(indices, QApplication.keyboardModifiers())
 
     def select_indices(self, indices, modifiers=Qt.NoModifier):
         if self.data is None:
@@ -1083,8 +1089,8 @@ def colors(data, variable, palette=None):
         elif variable.is_continuous:
             palette = colorpalette.ColorPaletteBW()
             palette = colorpalette.ContinuousPaletteGenerator(
-                QtGui.QColor(220, 220, 220),
-                QtGui.QColor(0, 0, 0),
+                QColor(220, 220, 220),
+                QColor(0, 0, 0),
                 False
             )
         else:
@@ -1141,13 +1147,13 @@ class mdsplotutils(plotutils):
 
     plotstyle = namespace(
         selected_pen=make_pen(Qt.yellow, width=3, cosmetic=True),
-        highligh_pen=QtGui.QPen(Qt.blue, 1),
+        highligh_pen=QPen(Qt.blue, 1),
         selected_brush=None,
-        default_color=QtGui.QColor(Qt.darkGray).rgba(),
+        default_color=QColor(Qt.darkGray).rgba(),
         discrete_palette=colorpalette.ColorPaletteGenerator(),
         continuous_palette=colorpalette.ContinuousPaletteGenerator(
-            QtGui.QColor(220, 220, 220),
-            QtGui.QColor(0, 0, 0),
+            QColor(220, 220, 220),
+            QColor(0, 0, 0),
             False
         ),
         symbols=ScatterPlotItem.Symbols,
@@ -1202,7 +1208,7 @@ class mdsplotutils(plotutils):
             plotstyle = mdsplotutils.plotstyle
 
         pens = numpy.array(
-            [mdsplotutils.make_pen(QtGui.QColor(*rgba), width=1)
+            [mdsplotutils.make_pen(QColor(*rgba), width=1)
              for rgba in basecolors],
             dtype=object)
 
@@ -1225,7 +1231,7 @@ class mdsplotutils(plotutils):
             plotstyle = mdsplotutils.plotstyle
 
         brush = numpy.array(
-            [mdsplotutils.make_brush(QtGui.QColor(*c))
+            [mdsplotutils.make_brush(QColor(*c))
              for c in basecolors],
             dtype=object)
 
@@ -1235,7 +1241,7 @@ class mdsplotutils(plotutils):
         fill_mask = flags & mdsplotutils.Filled
 
         if not numpy.all(fill_mask):
-            brush[~fill_mask] = QtGui.QBrush(Qt.NoBrush)
+            brush[~fill_mask] = QBrush(Qt.NoBrush)
         return brush
 
     @staticmethod
@@ -1320,7 +1326,7 @@ class mdsplotutils(plotutils):
                 colors = [(palette[i], "o", name)
                           for i, name in enumerate(color_var.values)]
             if shape_var is not None:
-                shapes = [(QtGui.QColor(Qt.gray),
+                shapes = [(QColor(Qt.gray),
                            symbols[i % (len(symbols) - 1)], name)
                           for i, name in enumerate(shape_var.values)]
             items = colors + shapes
@@ -1329,21 +1335,20 @@ class mdsplotutils(plotutils):
 
     @staticmethod
     def make_pen(color, width=1, cosmetic=True):
-        pen = QtGui.QPen(color)
+        pen = QPen(color)
         pen.setWidthF(width)
         pen.setCosmetic(cosmetic)
         return pen
 
     @staticmethod
     def make_brush(color, ):
-        return QtGui.QBrush(color, )
+        return QBrush(color, )
 
 
 def main_test(argv=sys.argv):
     import gc
-    argv = list(argv)
-    app = QtGui.QApplication(argv)
-
+    app = QApplication(list(argv))
+    argv = app.arguments()
     if len(argv) > 1:
         filename = argv[1]
     else:
