@@ -4,19 +4,28 @@ import math
 from itertools import chain
 import numpy as np
 
-from PyQt4 import QtCore
-from PyQt4 import QtGui
+from AnyQt.QtWidgets import (
+    QGraphicsView, QGraphicsScene, QGraphicsItem, QGraphicsSimpleTextItem,
+    QGraphicsTextItem, QGraphicsItemGroup, QGraphicsLineItem,
+    QGraphicsPathItem, QGraphicsRectItem, QSizePolicy
+)
+from AnyQt.QtGui import QPen, QColor, QBrush, QPainterPath, QPainter, QFont
+from AnyQt.QtCore import Qt, QEvent, QRectF, QSize
 
 import scipy.special
+from scipy.stats import f_oneway, chisquare
 
 import Orange.data
+from Orange.data.filter import FilterDiscrete, FilterContinuous, Values
 from Orange.statistics import contingency, distribution
 
 from Orange.widgets import widget, gui
 from Orange.widgets.settings import (Setting, DomainContextHandler,
                                      ContextSetting)
-from Orange.widgets.utils import datacaching
 from Orange.widgets.utils.itemmodels import VariableListModel
+from Orange.widgets.utils.annotated_data import (create_annotated_table,
+                                                 ANNOTATED_DATA_SIGNAL_NAME)
+from Orange.widgets.widget import Default
 
 
 def compute_scale(min_, max_):
@@ -36,7 +45,7 @@ def compute_scale(min_, max_):
 
 
 class BoxData:
-    def __init__(self, dist):
+    def __init__(self, dist, attr, group_val_index=None, group_var=None):
         self.dist = dist
         self.n = n = np.sum(dist[1])
         if n == 0:
@@ -59,8 +68,22 @@ class BoxData:
                     q.append(float(dist[0, i]))
                 thresh_i += 1
                 if thresh_i == 3:
+                    self.q25, self.median, self.q75 = q
                     break
-        self.q25, self.median, self.q75, *_ = q + [-1] * 3
+        else:
+            self.q25 = self.q75 = None
+            self.median = q[1] if len(q) == 2 else None
+        self.conditions = [FilterContinuous(attr, FilterContinuous.Between,
+                                            self.q25, self.q75)]
+        if group_val_index is not None:
+            self.conditions.append(FilterDiscrete(group_var, [group_val_index]))
+
+
+class FilterGraphicsRectItem(QGraphicsRectItem):
+    def __init__(self, conditions, *args):
+        super().__init__(*args)
+        self.filter = Values(conditions) if conditions else None
+        self.setFlag(QGraphicsItem.ItemIsSelectable)
 
 
 class OWBoxPlot(widget.OWWidget):
@@ -68,12 +91,14 @@ class OWBoxPlot(widget.OWWidget):
     Here's how the widget's functions call each other:
 
     - `set_data` is a signal handler fills the list boxes and calls
-    `attr_changed`.
+    `grouping_changed`.
 
-    - `attr_changed` handles changes of attribute or grouping (callbacks for
-    list boxes). It recomputes box data by calling `compute_box_data`, shows
-    the appropriate display box (discrete/continuous) and then calls
-    `layout_changed`
+    - `grouping_changed` handles changes of grouping attribute: it enables or
+    disables the box for ordering, orders attributes and calls `attr_changed`.
+
+    - `attr_changed` handles changes of attribute. It recomputes box data by
+    calling `compute_box_data`, shows the appropriate display box
+    (discrete/continuous) and then calls`layout_changed`
 
     - `layout_changed` constructs all the elements for the scene (as lists of
     QGraphicsItemGroup) and calls `display_changed`. It is called when the
@@ -93,46 +118,51 @@ class OWBoxPlot(widget.OWWidget):
     icon = "icons/BoxPlot.svg"
     priority = 100
     inputs = [("Data", Orange.data.Table, "set_data")]
+    outputs = [("Selected Data", Orange.data.Table, Default),
+               (ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)]
 
     #: Comparison types for continuous variables
     CompareNone, CompareMedians, CompareMeans = 0, 1, 2
 
     settingsHandler = DomainContextHandler()
+    conditions = ContextSetting([])
 
     attribute = ContextSetting(None)
+    order_by_importance = Setting(False)
     group_var = ContextSetting(None)
     show_annotations = Setting(True)
-    compare = Setting(CompareMedians)
+    compare = Setting(CompareMeans)
     stattest = Setting(0)
     sig_threshold = Setting(0.05)
     stretched = Setting(True)
+    auto_commit = Setting(True)
 
     _sorting_criteria_attrs = {
         CompareNone: "", CompareMedians: "median", CompareMeans: "mean"
     }
 
-    _pen_axis_tick = QtGui.QPen(QtCore.Qt.white, 5)
-    _pen_axis = QtGui.QPen(QtCore.Qt.darkGray, 3)
-    _pen_median = QtGui.QPen(QtGui.QBrush(QtGui.QColor(0xff, 0xff, 0x00)), 2)
-    _pen_paramet = QtGui.QPen(QtGui.QBrush(QtGui.QColor(0x33, 0x00, 0xff)), 2)
-    _pen_dotted = QtGui.QPen(QtGui.QBrush(QtGui.QColor(0x33, 0x00, 0xff)), 1)
-    _pen_dotted.setStyle(QtCore.Qt.DotLine)
-    _post_line_pen = QtGui.QPen(QtCore.Qt.lightGray, 2)
-    _post_grp_pen = QtGui.QPen(QtCore.Qt.lightGray, 4)
+    _pen_axis_tick = QPen(Qt.white, 5)
+    _pen_axis = QPen(Qt.darkGray, 3)
+    _pen_median = QPen(QBrush(QColor(0xff, 0xff, 0x00)), 2)
+    _pen_paramet = QPen(QBrush(QColor(0x33, 0x00, 0xff)), 2)
+    _pen_dotted = QPen(QBrush(QColor(0x33, 0x00, 0xff)), 1)
+    _pen_dotted.setStyle(Qt.DotLine)
+    _post_line_pen = QPen(Qt.lightGray, 2)
+    _post_grp_pen = QPen(Qt.lightGray, 4)
     for pen in (_pen_paramet, _pen_median, _pen_dotted,
                 _pen_axis, _pen_axis_tick, _post_line_pen, _post_grp_pen):
         pen.setCosmetic(True)
-        pen.setCapStyle(QtCore.Qt.RoundCap)
-        pen.setJoinStyle(QtCore.Qt.RoundJoin)
-    _pen_axis_tick.setCapStyle(QtCore.Qt.FlatCap)
+        pen.setCapStyle(Qt.RoundCap)
+        pen.setJoinStyle(Qt.RoundJoin)
+    _pen_axis_tick.setCapStyle(Qt.FlatCap)
 
-    _box_brush = QtGui.QBrush(QtGui.QColor(0x33, 0x88, 0xff, 0xc0))
+    _box_brush = QBrush(QColor(0x33, 0x88, 0xff, 0xc0))
 
-    _axis_font = QtGui.QFont()
+    _axis_font = QFont()
     _axis_font.setPixelSize(12)
-    _label_font = QtGui.QFont()
+    _label_font = QFont()
     _label_font.setPixelSize(11)
-    _attr_brush = QtGui.QBrush(QtGui.QColor(0x33, 0x00, 0xff))
+    _attr_brush = QBrush(QColor(0x33, 0x00, 0xff))
 
     graph_name = "box_scene"
 
@@ -148,19 +178,34 @@ class OWBoxPlot(widget.OWWidget):
         self.scale_x = self.scene_min_x = self.scene_width = 0
         self.label_width = 0
 
-        common_options = dict(
-            callback=self.attr_changed, sizeHint=(200, 100))
         self.attrs = VariableListModel()
-        gui.listView(
+        view = gui.listView(
             self.controlArea, self, "attribute", box="Variable",
-            model=self.attrs, **common_options)
+            model=self.attrs, callback=self.attr_changed)
+        view.setMinimumSize(QSize(30, 30))
+        # Any other policy than Ignored will let the QListBox's scrollbar
+        # set the minimal height (see the penultimate paragraph of
+        # http://doc.qt.io/qt-4.8/qabstractscrollarea.html#addScrollBarWidget)
+        view.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Ignored)
+        gui.separator(view.box, 6, 6)
+        self.cb_order = gui.checkBox(
+            view.box, self, "order_by_importance",
+            "Order by relevance",
+            tooltip="Order by 𝜒² or ANOVA over the subgroups",
+            callback=self.apply_sorting)
         self.group_vars = VariableListModel()
-        gui.listView(
-            self.controlArea, self, "group_var", box="Grouping",
-            model=self.group_vars, **common_options)
+        view = gui.listView(
+            self.controlArea, self, "group_var", box="Subgroups",
+            model=self.group_vars, callback=self.grouping_changed)
+        view.setMinimumSize(QSize(30, 30))
+        # See the comment above
+        view.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Ignored)
 
         # TODO: move Compare median/mean to grouping box
-        self.display_box = gui.vBox(self.controlArea, "Display")
+        # The vertical size policy is needed to let only the list views expand
+        self.display_box = gui.vBox(
+            self.controlArea, "Display",
+            sizePolicy=(QSizePolicy.Minimum, QSizePolicy.Maximum))
 
         gui.checkBox(self.display_box, self, "show_annotations", "Annotate",
                      callback=self.display_changed)
@@ -169,32 +214,41 @@ class OWBoxPlot(widget.OWWidget):
             btnLabels=["No comparison", "Compare medians", "Compare means"],
             callback=self.display_changed)
 
+        # The vertical size policy is needed to let only the list views expand
         self.stretching_box = gui.checkBox(
             self.controlArea, self, 'stretched', "Stretch bars", box='Display',
-            callback=self.display_changed).box
+            callback=self.display_changed,
+            sizePolicy=(QSizePolicy.Minimum, QSizePolicy.Maximum)).box
+
+        gui.auto_commit(self.controlArea, self, "auto_commit",
+                        "Send Selection", "Send Automatically")
 
         gui.vBox(self.mainArea, addSpace=True)
-        self.box_scene = QtGui.QGraphicsScene()
-        self.box_view = QtGui.QGraphicsView(self.box_scene)
-        self.box_view.setRenderHints(QtGui.QPainter.Antialiasing |
-                                     QtGui.QPainter.TextAntialiasing |
-                                     QtGui.QPainter.SmoothPixmapTransform)
+        self.box_scene = QGraphicsScene()
+        self.box_scene.selectionChanged.connect(self.commit)
+        self.box_view = QGraphicsView(self.box_scene)
+        self.box_view.setRenderHints(QPainter.Antialiasing |
+                                     QPainter.TextAntialiasing |
+                                     QPainter.SmoothPixmapTransform)
         self.box_view.viewport().installEventFilter(self)
 
         self.mainArea.layout().addWidget(self.box_view)
 
         e = gui.hBox(self.mainArea, addSpace=False)
         self.infot1 = gui.widgetLabel(e, "<center>No test results.</center>")
-        self.mainArea.setMinimumWidth(650)
+        self.mainArea.setMinimumWidth(600)
 
         self.stats = self.dist = self.conts = []
         self.is_continuous = False
 
         self.update_display_box()
 
+    def sizeHint(self):
+        return QSize(100, 500)  # Vertical size is regulated by mainArea
+
     def eventFilter(self, obj, event):
         if obj is self.box_view.viewport() and \
-                event.type() == QtCore.QEvent.Resize:
+                event.type() == QEvent.Resize:
             self.layout_changed()
 
         return super().eventFilter(obj, event)
@@ -224,13 +278,79 @@ class OWBoxPlot(widget.OWWidget):
             else:
                 self.group_var = None  # Reset to trigger selection via callback
             self.openContext(self.dataset)
-            self.attr_changed()
+            self.grouping_changed()
         else:
             self.reset_all_data()
+        self.commit()
+
+    def apply_sorting(self):
+        def compute_score(attr):
+            if attr is group_var:
+                return 3
+            if attr.is_continuous:
+                # One-way ANOVA
+                col = data.get_column_view(attr)[0].astype(float)
+                groups = (col[group_col == i] for i in range(n_groups))
+                groups = (col[~np.isnan(col)] for col in groups)
+                groups = [group for group in groups if len(group)]
+                p = f_oneway(*groups)[1] if len(groups) > 1 else 2
+            else:
+                # Chi-square with the given distribution into groups
+                # (see degrees of freedom in computation of the p-value)
+                if not attr.values or not group_var.values:
+                    return 2
+                observed = np.array(
+                    contingency.get_contingency(data, group_var, attr))
+                observed = observed[observed.sum(axis=1) != 0, :]
+                observed = observed[:, observed.sum(axis=0) != 0]
+                if min(observed.shape) < 2:
+                    return 2
+                expected = \
+                    np.outer(observed.sum(axis=1), observed.sum(axis=0)) / \
+                    np.sum(observed)
+                p = chisquare(observed.ravel(), f_exp=expected.ravel(),
+                              ddof=n_groups - 1)[1]
+            if math.isnan(p):
+                return 2
+            return p
+
+        data = self.dataset
+        if data is None:
+            return
+        domain = data.domain
+        attribute = self.attribute
+        group_var = self.group_var
+        if self.order_by_importance and group_var is not None:
+            n_groups = len(group_var.values)
+            group_col = data.get_column_view(group_var)[0] if \
+                domain.has_continuous_attributes(
+                    include_class=True, include_metas=True) else None
+            self.attrs.sort(key=compute_score)
+        else:
+            self.attrs[:] = chain(
+                domain.variables,
+                (a for a in data.domain.metas if a.is_primitive()))
+        self.attribute = attribute
 
     def reset_all_data(self):
         self.clear_scene()
         self.infot1.setText("")
+        self.attrs[:] = []
+        self.group_vars[:] = []
+        self.is_continuous = False
+        self.update_display_box()
+
+    def grouping_changed(self):
+        self.cb_order.setEnabled(self.group_var is not None)
+        self.apply_sorting()
+        self.attr_changed()
+
+    def select_box_items(self):
+        temp_cond = self.conditions.copy()
+        for box in self.box_scene.items():
+            if isinstance(box, FilterGraphicsRectItem):
+                box.setSelected(box.filter.conditions in
+                                [c.conditions for c in temp_cond])
 
     def attr_changed(self):
         self.compute_box_data()
@@ -250,24 +370,24 @@ class OWBoxPlot(widget.OWWidget):
         if not attr:
             return
         dataset = self.dataset
-        if dataset is None:
+        self.is_continuous = attr.is_continuous
+        if dataset is None or not self.is_continuous and not attr.values or \
+                        self.group_var and not self.group_var.values:
             self.stats = self.dist = self.conts = []
             return
-        self.is_continuous = attr.is_continuous
         if self.group_var:
             self.dist = []
-            self.conts = datacaching.getCached(
-                dataset, contingency.get_contingency,
-                (dataset, attr, self.group_var))
+            self.conts = contingency.get_contingency(
+                dataset, attr, self.group_var)
             if self.is_continuous:
-                self.stats = [BoxData(cont) for cont in self.conts]
+                self.stats = [BoxData(cont, attr, i, self.group_var)
+                              for i, cont in enumerate(self.conts)]
             self.label_txts_all = self.group_var.values
         else:
-            self.dist = datacaching.getCached(
-                dataset, distribution.get_distribution, (dataset, attr))
+            self.dist = distribution.get_distribution(dataset, attr)
             self.conts = []
             if self.is_continuous:
-                self.stats = [BoxData(self.dist)]
+                self.stats = [BoxData(self.dist, attr, None)]
             self.label_txts_all = [""]
         self.label_txts = [txts for stat, txts in zip(self.stats,
                                                       self.label_txts_all)
@@ -284,12 +404,15 @@ class OWBoxPlot(widget.OWWidget):
             self.display_box.hide()
 
     def clear_scene(self):
+        self.closeContext()
+        self.box_scene.clearSelection()
         self.box_scene.clear()
         self.attr_labels = []
         self.labels = []
         self.boxes = []
         self.mean_labels = []
         self.posthoc_lines = []
+        self.openContext(self.dataset)
 
     def layout_changed(self):
         attr = self.attribute
@@ -308,9 +431,9 @@ class OWBoxPlot(widget.OWWidget):
         self.boxes = [self.box_group(stat) for stat in self.stats]
         self.labels = [self.label_group(stat, attr, mean_lab)
                        for stat, mean_lab in zip(self.stats, self.mean_labels)]
-        self.attr_labels = [QtGui.QGraphicsSimpleTextItem(lab)
+        self.attr_labels = [QGraphicsSimpleTextItem(lab)
                             for lab in self.label_txts]
-        for it in chain(self.labels, self.boxes, self.attr_labels):
+        for it in chain(self.labels, self.attr_labels):
             self.box_scene.addItem(it)
         self.display_changed()
 
@@ -331,7 +454,9 @@ class OWBoxPlot(widget.OWWidget):
 
         for row, box_index in enumerate(self.order):
             y = (-len(self.stats) + row) * heights + 10
-            self.boxes[box_index].setY(y)
+            for item in self.boxes[box_index].childItems():
+                self.box_scene.addItem(item)
+                item.setY(y)
             labels = self.labels[box_index]
 
             if self.show_annotations:
@@ -347,39 +472,41 @@ class OWBoxPlot(widget.OWWidget):
             else:
                 stat = self.stats[box_index]
 
-                if self.compare == OWBoxPlot.CompareMedians:
+                if self.compare == OWBoxPlot.CompareMedians and \
+                        stat.median is not None:
                     pos = stat.median + 5 / self.scale_x
-                elif self.compare == OWBoxPlot.CompareMeans:
+                elif self.compare == OWBoxPlot.CompareMeans or stat.q25 is None:
                     pos = stat.mean + 5 / self.scale_x
                 else:
                     pos = stat.q25
                 label.setX(pos * self.scale_x)
                 label.show()
 
-        r = QtCore.QRectF(self.scene_min_x, -30 - len(self.stats) * heights,
-                          self.scene_width, len(self.stats) * heights + 90)
+        r = QRectF(self.scene_min_x, -30 - len(self.stats) * heights,
+                   self.scene_width, len(self.stats) * heights + 90)
         self.box_scene.setSceneRect(r)
 
         self.compute_tests()
         self.show_posthoc()
+        self.select_box_items()
 
     def display_changed_disc(self):
         self.clear_scene()
-        self.attr_labels = [QtGui.QGraphicsSimpleTextItem(lab)
+        self.attr_labels = [QGraphicsSimpleTextItem(lab)
                             for lab in self.label_txts_all]
 
         if not self.stretched:
             if self.group_var:
                 self.labels = [
-                    QtGui.QGraphicsTextItem("{}".format(int(sum(cont))))
+                    QGraphicsTextItem("{}".format(int(sum(cont))))
                     for cont in self.conts]
             else:
                 self.labels = [
-                    QtGui.QGraphicsTextItem(str(int(sum(self.dist))))]
+                    QGraphicsTextItem(str(int(sum(self.dist))))]
 
         self.draw_axis_disc()
         if self.group_var:
-            self.boxes = [self.strudel(cont) for cont in self.conts]
+            self.boxes = [self.strudel(cont, i) for i, cont in enumerate(self.conts)]
         else:
             self.boxes = [self.strudel(self.dist)]
 
@@ -403,19 +530,21 @@ class OWBoxPlot(widget.OWWidget):
             if self.attribute is not self.group_var:
                 for text_item, bar_part in zip(box.childItems()[1::2],
                                                box.childItems()[::2]):
-                    label = QtGui.QGraphicsSimpleTextItem(
+                    label = QGraphicsSimpleTextItem(
                         text_item.toPlainText())
                     label.setPos(bar_part.boundingRect().x(),
                                  y - label.boundingRect().height() - 8)
                     self.box_scene.addItem(label)
             for text_item in box.childItems()[1::2]:
                 box.removeFromGroup(text_item)
-            self.box_scene.addItem(box)
-            box.setPos(0, y)
+            for item in box.childItems():
+                self.box_scene.addItem(item)
+                item.setPos(0, y)
         self.box_scene.setSceneRect(-self.label_width - 5,
                                     -30 - len(self.boxes) * 40,
                                     self.scene_width, len(self.boxes * 40) + 90)
         self.infot1.setText("")
+        self.select_box_items()
 
     # noinspection PyPep8Naming
     def compute_tests(self):
@@ -428,6 +557,8 @@ class OWBoxPlot(widget.OWWidget):
             df = pooled_var ** 2 / \
                 ((d1.var / d1.n) ** 2 / (d1.n - 1) +
                  (d2.var / d2.n) ** 2 / (d2.n - 1))
+            if pooled_var == 0:
+                return np.nan, np.nan
             t = abs(d1.mean - d2.mean) / math.sqrt(pooled_var)
             p = 2 * (1 - scipy.special.stdtr(df, t))
             return t, p
@@ -472,20 +603,20 @@ class OWBoxPlot(widget.OWWidget):
         self.infot1.setText("<center>%s</center>" % t)
 
     def mean_label(self, stat, attr, val_name):
-        label = QtGui.QGraphicsItemGroup()
-        t = QtGui.QGraphicsSimpleTextItem(
+        label = QGraphicsItemGroup()
+        t = QGraphicsSimpleTextItem(
             "%.*f" % (attr.number_of_decimals + 1, stat.mean), label)
         t.setFont(self._label_font)
         bbox = t.boundingRect()
         w2, h = bbox.width() / 2, bbox.height()
         t.setPos(-w2, -h)
-        tpm = QtGui.QGraphicsSimpleTextItem(
+        tpm = QGraphicsSimpleTextItem(
             " \u00b1 " + "%.*f" % (attr.number_of_decimals + 1, stat.dev),
             label)
         tpm.setFont(self._label_font)
         tpm.setPos(w2, -h)
         if val_name:
-            vnm = QtGui.QGraphicsSimpleTextItem(val_name + ": ", label)
+            vnm = QGraphicsSimpleTextItem(val_name + ": ", label)
             vnm.setFont(self._label_font)
             vnm.setBrush(self._attr_brush)
             vb = vnm.boundingRect()
@@ -498,7 +629,7 @@ class OWBoxPlot(widget.OWWidget):
     def draw_axis(self):
         """Draw the horizontal axis and sets self.scale_x"""
         misssing_stats = not self.stats
-        stats = self.stats or [BoxData(np.array([[0.], [1.]]))]
+        stats = self.stats or [BoxData(np.array([[0.], [1.]]), self.attribute)]
         mean_labels = self.mean_labels or [self.mean_label(stats[0], self.attribute, "")]
         bottom = min(stat.a_min for stat in stats)
         top = max(stat.a_max for stat in stats)
@@ -538,7 +669,7 @@ class OWBoxPlot(widget.OWWidget):
                 self.attribute.repr_val(val) if not misssing_stats else "?",
                 self._axis_font)
             t.setFlags(
-                t.flags() | QtGui.QGraphicsItem.ItemIgnoresTransformations)
+                t.flags() | QGraphicsItem.ItemIgnoresTransformations)
             r = t.boundingRect()
             t.setPos(val * scale_x - r.width() / 2, 8)
             if val >= top:
@@ -604,7 +735,7 @@ class OWBoxPlot(widget.OWWidget):
 
     def label_group(self, stat, attr, mean_lab):
         def centered_text(val, pos):
-            t = QtGui.QGraphicsSimpleTextItem(
+            t = QGraphicsSimpleTextItem(
                 "%.*f" % (attr.number_of_decimals + 1, val), labels)
             t.setFont(self._label_font)
             bbox = t.boundingRect()
@@ -612,58 +743,60 @@ class OWBoxPlot(widget.OWWidget):
             return t
 
         def line(x, down=1):
-            QtGui.QGraphicsLineItem(x, 12 * down, x, 20 * down, labels)
+            QGraphicsLineItem(x, 12 * down, x, 20 * down, labels)
 
         def move_label(label, frm, to):
             label.setX(to)
             to += t_box.width() / 2
-            path = QtGui.QPainterPath()
+            path = QPainterPath()
             path.lineTo(0, 4)
             path.lineTo(to - frm, 4)
             path.lineTo(to - frm, 8)
-            p = QtGui.QGraphicsPathItem(path)
+            p = QGraphicsPathItem(path)
             p.setPos(frm, 12)
             labels.addToGroup(p)
 
-        labels = QtGui.QGraphicsItemGroup()
+        labels = QGraphicsItemGroup()
 
         labels.addToGroup(mean_lab)
         m = stat.mean * self.scale_x
         mean_lab.setPos(m, -22)
         line(m, -1)
 
-        msc = stat.median * self.scale_x
-        med_t = centered_text(stat.median, msc)
-        med_box_width2 = med_t.boundingRect().width()
-        line(msc)
+        if stat.median is not None:
+            msc = stat.median * self.scale_x
+            med_t = centered_text(stat.median, msc)
+            med_box_width2 = med_t.boundingRect().width()
+            line(msc)
 
-        x = stat.q25 * self.scale_x
-        t = centered_text(stat.q25, x)
-        t_box = t.boundingRect()
-        med_left = msc - med_box_width2
-        if x + t_box.width() / 2 >= med_left - 5:
-            move_label(t, x, med_left - t_box.width() - 5)
-        else:
-            line(x)
+        if stat.q25 is not None:
+            x = stat.q25 * self.scale_x
+            t = centered_text(stat.q25, x)
+            t_box = t.boundingRect()
+            med_left = msc - med_box_width2
+            if x + t_box.width() / 2 >= med_left - 5:
+                move_label(t, x, med_left - t_box.width() - 5)
+            else:
+                line(x)
 
-        x = stat.q75 * self.scale_x
-        t = centered_text(stat.q75, x)
-        t_box = t.boundingRect()
-        med_right = msc + med_box_width2
-        if x - t_box.width() / 2 <= med_right + 5:
-            move_label(t, x, med_right + 5)
-        else:
-            line(x)
+        if stat.q75 is not None:
+            x = stat.q75 * self.scale_x
+            t = centered_text(stat.q75, x)
+            t_box = t.boundingRect()
+            med_right = msc + med_box_width2
+            if x - t_box.width() / 2 <= med_right + 5:
+                move_label(t, x, med_right + 5)
+            else:
+                line(x)
 
         return labels
 
     def box_group(self, stat, height=20):
         def line(x0, y0, x1, y1, *args):
-            return QtGui.QGraphicsLineItem(x0 * scale_x, y0, x1 * scale_x, y1,
-                                           *args)
+            return QGraphicsLineItem(x0 * scale_x, y0, x1 * scale_x, y1, *args)
 
         scale_x = self.scale_x
-        box = QtGui.QGraphicsItemGroup()
+        box = QGraphicsItemGroup()
         whisker1 = line(stat.a_min, -1.5, stat.a_min, 1.5, box)
         whisker2 = line(stat.a_max, -1.5, stat.a_max, 1.5, box)
         vert_line = line(stat.a_min, 0, stat.a_max, 0, box)
@@ -674,26 +807,31 @@ class OWBoxPlot(widget.OWWidget):
         var_line = line(stat.mean - stat.dev, 0, stat.mean + stat.dev, 0, box)
         var_line.setPen(self._pen_paramet)
 
-        mbox = QtGui.QGraphicsRectItem(stat.q25 * scale_x, -height / 2,
-                                       (stat.q75 - stat.q25) * scale_x, height,
-                                       box)
-        mbox.setBrush(self._box_brush)
-        mbox.setPen(QtGui.QPen(QtCore.Qt.NoPen))
-        mbox.setZValue(-200)
+        if stat.q25 is not None and stat.q75 is not None:
+            mbox = FilterGraphicsRectItem(
+                stat.conditions, stat.q25 * scale_x, -height / 2,
+                (stat.q75 - stat.q25) * scale_x, height, box)
+            mbox.setBrush(self._box_brush)
+            mbox.setPen(QPen(Qt.NoPen))
+            mbox.setZValue(-200)
 
-        median_line = line(stat.median, -height / 2,
-                           stat.median, height / 2, box)
-        median_line.setPen(self._pen_median)
-        median_line.setZValue(-150)
+        if stat.median is not None:
+            median_line = line(stat.median, -height / 2,
+                               stat.median, height / 2, box)
+            median_line.setPen(self._pen_median)
+            median_line.setZValue(-150)
 
         return box
 
-    def strudel(self, dist):
+    def strudel(self, dist, group_val_index=None):
         attr = self.attribute
         ss = np.sum(dist)
-        box = QtGui.QGraphicsItemGroup()
+        box = QGraphicsItemGroup()
         if ss < 1e-6:
-            QtGui.QGraphicsRectItem(0, -10, 1, 10, box)
+            cond = [FilterDiscrete(attr, None)]
+            if group_val_index is not None:
+                cond.append(FilterDiscrete(self.group_var, [group_val_index]))
+            FilterGraphicsRectItem(cond, 0, -10, 1, 10, box)
         cum = 0
         for i, v in enumerate(dist):
             if v < 1e-6:
@@ -701,20 +839,35 @@ class OWBoxPlot(widget.OWWidget):
             if self.stretched:
                 v /= ss
             v *= self.scale_x
-            rect = QtGui.QGraphicsRectItem(cum + 1, -6, v - 2, 12, box)
-            rect.setBrush(QtGui.QBrush(QtGui.QColor(*attr.colors[i])))
-            rect.setPen(QtGui.QPen(QtCore.Qt.NoPen))
+            cond = [FilterDiscrete(attr, [i])]
+            if group_val_index is not None:
+                cond.append(FilterDiscrete(self.group_var, [group_val_index]))
+            rect = FilterGraphicsRectItem(cond, cum + 1, -6, v - 2, 12, box)
+            rect.setBrush(QBrush(QColor(*attr.colors[i])))
+            rect.setPen(QPen(Qt.NoPen))
             if self.stretched:
                 tooltip = "{}: {:.2f}%".format(attr.values[i],
                                                100 * dist[i] / sum(dist))
             else:
                 tooltip = "{}: {}".format(attr.values[i], int(dist[i]))
             rect.setToolTip(tooltip)
-            text = QtGui.QGraphicsTextItem(attr.values[i])
+            text = QGraphicsTextItem(attr.values[i])
             box.addToGroup(text)
             cum += v
         return box
 
+    def commit(self):
+        self.conditions = [item.filter for item in
+                           self.box_scene.selectedItems() if item.filter]
+        selected, selection = None, []
+        if self.conditions:
+            selected = Values(self.conditions, conjunction=False)(self.dataset)
+            selection = [i for i, inst in enumerate(self.dataset)
+                         if inst in selected]
+        self.send("Selected Data", selected)
+        self.send(ANNOTATED_DATA_SIGNAL_NAME,
+                  create_annotated_table(self.dataset, selection))
+
     def show_posthoc(self):
         def line(y0, y1):
             it = self.box_scene.addLine(x, y0, x, y1, self._post_line_pen)
@@ -783,14 +936,15 @@ class OWBoxPlot(widget.OWWidget):
 
 
 def main(argv=None):
+    from AnyQt.QtWidgets import QApplication
     if argv is None:
         argv = sys.argv
     argv = list(argv)
-    app = QtGui.QApplication(argv)
+    app = QApplication(argv)
     if len(argv) > 1:
         filename = argv[1]
     else:
-        filename = "brown-selected"
+        filename = "heart_disease"
 
     data = Orange.data.Table(filename)
     w = OWBoxPlot()
