@@ -1,7 +1,11 @@
 import numpy as np
-from PyQt4.QtCore import Qt, QTimer
-from PyQt4 import QtGui
-from PyQt4.QtGui import QApplication
+
+from AnyQt.QtCore import QSize, Qt, QTimer
+from AnyQt.QtGui import (
+    QPen, QFont, QFontInfo, QPalette, QKeySequence,
+)
+from AnyQt.QtWidgets import QApplication, QAction
+
 from sklearn.neighbors import NearestNeighbors
 from sklearn.metrics import r2_score
 
@@ -14,14 +18,17 @@ from Orange.preprocess.score import ReliefF, RReliefF
 from Orange.widgets import gui
 from Orange.widgets.settings import \
     DomainContextHandler, Setting, ContextSetting, SettingProvider
+from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.visualize.owscatterplotgraph import OWScatterPlotGraph
 from Orange.widgets.visualize.utils import VizRankDialogAttrPair
 from Orange.widgets.widget import OWWidget, Default, AttributeList, Msg
+from Orange.widgets.utils.annotated_data import (create_annotated_table,
+                                                 ANNOTATED_DATA_SIGNAL_NAME)
 
 
 def font_resize(font, factor, minsize=None, maxsize=None):
-    font = QtGui.QFont(font)
-    fontinfo = QtGui.QFontInfo(font)
+    font = QFont(font)
+    fontinfo = QFontInfo(font)
     size = fontinfo.pointSizeF() * factor
 
     if minsize is not None:
@@ -34,7 +41,7 @@ def font_resize(font, factor, minsize=None, maxsize=None):
 
 
 class ScatterPlotVizRank(VizRankDialogAttrPair):
-    captionTitle = "Score plots"
+    captionTitle = "Score Plots"
     K = 10
 
     def check_preconditions(self):
@@ -57,9 +64,9 @@ class ScatterPlotVizRank(VizRankDialogAttrPair):
 
     def compute_score(self, state):
         graph = self.master.graph
-        ind12 = [graph.data_domain.index(self.attrs[x]) for x in state]
+        ind12 = [graph.domain.index(self.attrs[x]) for x in state]
         valid = graph.get_valid_list(ind12)
-        X = graph.scaled_data[ind12, :][:, valid].T
+        X = graph.jittered_data[ind12, :][:, valid].T
         Y = self.master.data.Y[valid]
         if X.shape[0] < self.K:
             return
@@ -67,13 +74,16 @@ class ScatterPlotVizRank(VizRankDialogAttrPair):
         knn = NearestNeighbors(n_neighbors=n_neighbors).fit(X)
         ind = knn.kneighbors(return_distance=False)
         if self.master.data.domain.has_discrete_class:
-            return -np.sum(Y[ind] == Y.reshape(-1, 1))
+            return -np.sum(Y[ind] == Y.reshape(-1, 1)) / n_neighbors / len(Y)
         else:
             return -r2_score(Y, np.mean(Y[ind], axis=1)) * \
                    (len(Y) / len(self.master.data))
 
+    def bar_length(self, score):
+        return max(0, -score)
+
     def score_heuristic(self):
-        X = self.master.graph.scaled_data.T
+        X = self.master.graph.jittered_data.T
         Y = self.master.data.Y
         mdomain = self.master.data.domain
         dom = Domain([ContinuousVariable(str(i)) for i in range(X.shape[1])],
@@ -82,23 +92,27 @@ class ScatterPlotVizRank(VizRankDialogAttrPair):
         relief = ReliefF if isinstance(dom.class_var, DiscreteVariable) \
             else RReliefF
         weights = relief(n_iterations=100, k_nearest=self.K)(data)
-        attrs = sorted(zip(weights, mdomain.attributes), reverse=True)
+        attrs = sorted(zip(weights, mdomain.attributes),
+                       key=lambda x: (-x[0], x[1].name))
         return [a for _, a in attrs]
 
 
 class OWScatterPlot(OWWidget):
+    """Scatterplot visualization with explorative analysis and intelligent
+    data visualization enhancements."""
+
     name = 'Scatter Plot'
     description = "Interactive scatter plot visualization with " \
                   "intelligent data visualization enhancements."
     icon = "icons/ScatterPlot.svg"
-    priority = 210
+    priority = 140
 
     inputs = [("Data", Table, "set_data", Default),
               ("Data Subset", Table, "set_subset_data"),
               ("Features", AttributeList, "set_shown_attributes")]
 
     outputs = [("Selected Data", Table, Default),
-               ("Other Data", Table),
+               (ANNOTATED_DATA_SIGNAL_NAME, Table),
                ("Features", Table)]
 
     settingsHandler = DomainContextHandler()
@@ -107,8 +121,8 @@ class OWScatterPlot(OWWidget):
     auto_sample = Setting(True)
     toolbar_selection = Setting(0)
 
-    attr_x = ContextSetting("")
-    attr_y = ContextSetting("")
+    attr_x = ContextSetting(None)
+    attr_y = ContextSetting(None)
 
     graph = SettingProvider(OWScatterPlotGraph)
 
@@ -127,7 +141,7 @@ class OWScatterPlot(OWWidget):
         box.layout().addWidget(self.graph.plot_widget)
         plot = self.graph.plot_widget
 
-        axispen = QtGui.QPen(self.palette().color(QtGui.QPalette.Text))
+        axispen = QPen(self.palette().color(QPalette.Text))
         axis = plot.getAxis("bottom")
         axis.setPen(axispen)
 
@@ -146,26 +160,24 @@ class OWScatterPlot(OWWidget):
             labelWidth=50, orientation=Qt.Horizontal, sendSelectedValue=True,
             valueType=str)
         box = gui.vBox(self.controlArea, "Axis Data")
-        self.cb_attr_x = gui.comboBox(box, self, "attr_x", label="Axis x:",
-                                      callback=self.update_attr,
-                                      **common_options)
-        self.cb_attr_y = gui.comboBox(box, self, "attr_y", label="Axis y:",
-                                      callback=self.update_attr,
-                                      **common_options)
+        dmod = DomainModel
+        self.xy_model = DomainModel(dmod.MIXED, valid_types=dmod.PRIMITIVE)
+        gui.comboBox(
+            box, self, "attr_x", label="Axis x:", callback=self.update_attr,
+            model=self.xy_model, **common_options)
+        self.cb_attr_y = gui.comboBox(
+            box, self, "attr_y", label="Axis y:", callback=self.update_attr,
+            model=self.xy_model, **common_options)
 
-        self.vizrank = ScatterPlotVizRank(self)
         vizrank_box = gui.hBox(box)
         gui.separator(vizrank_box, width=common_options["labelWidth"])
-        self.vizrank_button_tooltip = "Find informative projections"
-        self.vizrank_button = gui.button(
-            vizrank_box, self, "Score Plots", callback=self.vizrank.reshow,
-            tooltip=self.vizrank_button_tooltip, enabled=False)
-        self.vizrank.pairSelected.connect(self.set_attr)
+        self.vizrank, self.vizrank_button = ScatterPlotVizRank.add_vizrank(
+            vizrank_box, self, "Find Informative Projections", self.set_attr)
 
         gui.separator(box)
 
         gui.valueSlider(
-            box, self, value='graph.jitter_size',  label='Jittering: ',
+            box, self, value='graph.jitter_size', label='Jittering: ',
             values=self.jitter_sizes, callback=self.reset_graph_data,
             labelFormat=lambda x:
             "None" if x == 0 else ("%.1f %%" if x < 1 else "%d %%") % x)
@@ -179,26 +191,35 @@ class OWScatterPlot(OWWidget):
         self.sampling.setVisible(False)
 
         box = gui.vBox(self.controlArea, "Points")
+        self.color_model = DomainModel(
+            placeholder="(Same color)", valid_types=dmod.PRIMITIVE)
         self.cb_attr_color = gui.comboBox(
             box, self, "graph.attr_color", label="Color:",
-            emptyString="(Same color)", callback=self.update_colors,
-            **common_options)
+            callback=self.update_colors,
+            model=self.color_model, **common_options)
+        self.label_model = DomainModel(
+            placeholder="(No labels)", valid_types=dmod.PRIMITIVE)
         self.cb_attr_label = gui.comboBox(
             box, self, "graph.attr_label", label="Label:",
-            emptyString="(No labels)", callback=self.graph.update_labels,
-            **common_options)
+            callback=self.graph.update_labels,
+            model=self.label_model, **common_options)
+        self.shape_model = DomainModel(
+            placeholder="(Same shape)", valid_types=DiscreteVariable)
         self.cb_attr_shape = gui.comboBox(
             box, self, "graph.attr_shape", label="Shape:",
-            emptyString="(Same shape)", callback=self.graph.update_shapes,
-            **common_options)
+            callback=self.graph.update_shapes,
+            model=self.shape_model, **common_options)
+        self.size_model = DomainModel(
+            placeholder="(Same size)", valid_types=ContinuousVariable)
         self.cb_attr_size = gui.comboBox(
             box, self, "graph.attr_size", label="Size:",
-            emptyString="(Same size)", callback=self.graph.update_sizes,
-            **common_options)
+            callback=self.graph.update_sizes,
+            model=self.size_model, **common_options)
+        self.models = [self.xy_model, self.color_model, self.label_model,
+                       self.shape_model, self.size_model]
 
         g = self.graph.gui
-        box2 = g.point_properties_box(self.controlArea, box)
-
+        g.point_properties_box(self.controlArea, box)
         box = gui.vBox(self.controlArea, "Plot Properties")
         g.add_widgets([g.ShowLegend, g.ShowGridLines], box)
         gui.checkBox(
@@ -240,18 +261,18 @@ class OWScatterPlot(OWWidget):
             viewbox = plot.getViewBox()
             viewbox.autoRange()
 
-        zoom_in = QtGui.QAction(
+        zoom_in = QAction(
             "Zoom in", self, triggered=lambda: zoom(1.25)
         )
-        zoom_in.setShortcuts([QtGui.QKeySequence(QtGui.QKeySequence.ZoomIn),
-                              QtGui.QKeySequence(self.tr("Ctrl+="))])
-        zoom_out = QtGui.QAction(
-            "Zoom out", self, shortcut=QtGui.QKeySequence.ZoomOut,
+        zoom_in.setShortcuts([QKeySequence(QKeySequence.ZoomIn),
+                              QKeySequence(self.tr("Ctrl+="))])
+        zoom_out = QAction(
+            "Zoom out", self, shortcut=QKeySequence.ZoomOut,
             triggered=lambda: zoom(1 / 1.25)
         )
-        zoom_fit = QtGui.QAction(
+        zoom_fit = QAction(
             "Fit in view", self,
-            shortcut=QtGui.QKeySequence(Qt.ControlModifier | Qt.Key_0),
+            shortcut=QKeySequence(Qt.ControlModifier | Qt.Key_0),
             triggered=fit_to_view
         )
         self.addActions([zoom_in, zoom_out, zoom_fit])
@@ -276,6 +297,7 @@ class OWScatterPlot(OWWidget):
         self.update_graph()
 
     def set_data(self, data):
+        self.clear_messages()
         self.Information.sampled_sql.clear()
         self.__timer.stop()
         self.sampling.setVisible(False)
@@ -307,6 +329,7 @@ class OWScatterPlot(OWWidget):
         if not same_domain:
             self.init_attr_values()
         self.vizrank.initialize()
+        self.vizrank.attrs = self.data.domain.attributes if self.data is not None else []
         self.vizrank_button.setEnabled(
             self.data is not None and self.data.domain.class_var is not None
             and len(self.data.domain.attributes) > 1 and len(self.data) > 1)
@@ -315,9 +338,35 @@ class OWScatterPlot(OWWidget):
             self.vizrank_button.setToolTip(
                 "Data with a class variable is required.")
         else:
-            self.vizrank_button.setToolTip(self.vizrank_button_tooltip)
+            self.vizrank_button.setToolTip("")
         self.openContext(self.data)
 
+        def findvar(name, iterable):
+            """Find a Orange.data.Variable in `iterable` by name"""
+            for el in iterable:
+                if isinstance(el, Orange.data.Variable) and el.name == name:
+                    return el
+            else:
+                return None
+        # handle restored settings from  < 3.3.9 when attr_* were stored
+        # by name
+        if isinstance(self.attr_x, str):
+            self.attr_x = findvar(self.attr_x, self.xy_model)
+        if isinstance(self.attr_y, str):
+            self.attr_y = findvar(self.attr_y, self.xy_model)
+        if isinstance(self.graph.attr_label, str):
+            self.graph.attr_label = findvar(
+                self.graph.attr_label, self.label_model)
+        if isinstance(self.graph.attr_color, str):
+            self.graph.attr_color = findvar(
+                self.graph.attr_color, self.color_model)
+        if isinstance(self.graph.attr_shape, str):
+            self.graph.attr_shape = findvar(
+                self.graph.attr_shape, self.shape_model)
+        if isinstance(self.graph.attr_size, str):
+            self.graph.attr_size = findvar(
+                self.graph.attr_size, self.size_model)
+
     def add_data(self, time=0.4):
         if self.data and len(self.data) > 2000:
             return self.__timer.stop()
@@ -353,15 +402,16 @@ class OWScatterPlot(OWWidget):
                 self.warning("Data subset does not support large Sql tables")
                 subset_data = None
         self.subset_data = self.move_primitive_metas_to_X(subset_data)
+        self.controls.graph.alpha_value.setEnabled(subset_data is None)
 
     # called when all signals are received, so the graph is updated only once
     def handleNewSignals(self):
         self.graph.new_data(self.data_metas_X, self.subset_data)
         if self.attribute_selection_list and \
-                all(attr in self.graph.data_domain
+                all(attr in self.graph.domain
                     for attr in self.attribute_selection_list):
-            self.attr_x = self.attribute_selection_list[0].name
-            self.attr_y = self.attribute_selection_list[1].name
+            self.attr_x = self.attribute_selection_list[0]
+            self.attr_y = self.attribute_selection_list[1]
         self.attribute_selection_list = None
         self.update_graph()
         self.cb_class_density.setEnabled(self.graph.can_draw_density())
@@ -377,64 +427,19 @@ class OWScatterPlot(OWWidget):
         return self.attr_x, self.attr_y
 
     def init_attr_values(self):
-        self.cb_attr_x.clear()
-        self.attr_x = None
-        self.cb_attr_y.clear()
-        self.attr_y = None
-        self.cb_attr_color.clear()
-        self.cb_attr_color.addItem("(Same color)")
-        self.graph.attr_color = None
-        self.cb_attr_label.clear()
-        self.cb_attr_label.addItem("(No labels)")
-        self.graph.attr_label = None
-        self.cb_attr_shape.clear()
-        self.cb_attr_shape.addItem("(Same shape)")
+        domain = self.data and self.data.domain
+        for model in self.models:
+            model.set_domain(domain)
+        self.attr_x = self.xy_model[0] if self.xy_model else None
+        self.attr_y = self.xy_model[1] if len(self.xy_model) >= 2 \
+            else self.attr_x
+        self.graph.attr_color = domain and self.data.domain.class_var or None
         self.graph.attr_shape = None
-        self.cb_attr_size.clear()
-        self.cb_attr_size.addItem("(Same size)")
         self.graph.attr_size = None
-        if not self.data:
-            return
-
-        for var in self.data.domain.metas:
-            if not var.is_primitive():
-                self.cb_attr_label.addItem(self.icons[var], var.name)
-        for attr in self.data.domain.variables:
-            self.cb_attr_x.addItem(self.icons[attr], attr.name)
-            self.cb_attr_y.addItem(self.icons[attr], attr.name)
-            self.cb_attr_color.addItem(self.icons[attr], attr.name)
-            if attr.is_discrete:
-                self.cb_attr_shape.addItem(self.icons[attr], attr.name)
-            else:
-                self.cb_attr_size.addItem(self.icons[attr], attr.name)
-            self.cb_attr_label.addItem(self.icons[attr], attr.name)
-        for var in self.data.domain.metas:
-            if var.is_primitive():
-                self.cb_attr_x.addItem(self.icons[var], var.name)
-                self.cb_attr_y.addItem(self.icons[var], var.name)
-                self.cb_attr_color.addItem(self.icons[var], var.name)
-                if var.is_discrete:
-                    self.cb_attr_shape.addItem(self.icons[var], var.name)
-                else:
-                    self.cb_attr_size.addItem(self.icons[var], var.name)
-                self.cb_attr_label.addItem(self.icons[var], var.name)
-
-        self.attr_x = self.cb_attr_x.itemText(0)
-        if self.cb_attr_y.count() > 1:
-            self.attr_y = self.cb_attr_y.itemText(1)
-        else:
-            self.attr_y = self.cb_attr_y.itemText(0)
-
-        if self.data.domain.class_var:
-            self.graph.attr_color = self.data.domain.class_var.name
-        else:
-            self.graph.attr_color = ""
-        self.graph.attr_shape = ""
-        self.graph.attr_size = ""
-        self.graph.attr_label = ""
+        self.graph.attr_label = None
 
     def set_attr(self, attr_x, attr_y):
-        self.attr_x, self.attr_y = attr_x.name, attr_y.name
+        self.attr_x, self.attr_y = attr_x, attr_y
         self.update_attr()
 
     def update_attr(self):
@@ -451,7 +456,7 @@ class OWScatterPlot(OWWidget):
 
     def update_graph(self, reset_view=True, **_):
         self.graph.zoomStack = []
-        if not self.graph.have_data:
+        if self.graph.data is None:
             return
         self.graph.update_data(self.attr_x, self.attr_y, reset_view)
 
@@ -459,25 +464,18 @@ class OWScatterPlot(OWWidget):
         self.send_data()
 
     def send_data(self):
-        selected = unselected = None
+        selected = None
+        selection = None
         # TODO: Implement selection for sql data
         if isinstance(self.data, SqlTable):
-            selected = unselected = self.data
+            selected = self.data
         elif self.data is not None:
             selection = self.graph.get_selection()
-            if len(selection) == 0:
-                self.send("Selected Data", None)
-                self.send("Other Data", self.data)
-                return
-            selected = self.data[selection]
-            unselection = np.full(len(self.data), True, dtype=bool)
-            unselection[selection] = False
-            unselected = self.data[unselection]
+            if len(selection) > 0:
+                selected = self.data[selection]
         self.send("Selected Data", selected)
-        if unselected is None or len(unselected) == 0:
-            self.send("Other Data", None)
-        else:
-            self.send("Other Data", unselected)
+        self.send(ANNOTATED_DATA_SIGNAL_NAME,
+                  create_annotated_table(self.data, selection))
 
     def send_features(self):
         features = None
@@ -491,32 +489,22 @@ class OWScatterPlot(OWWidget):
         self.send_data()
         self.send_features()
 
-    def closeEvent(self, ce):
-        self.vizrank.close()
-        super().closeEvent(ce)
-
-    def hideEvent(self, he):
-        self.vizrank.hide()
-        super().hideEvent(he)
-
     def get_widget_name_extension(self):
         if self.data is not None:
-            return "{} vs {}".format(self.combo_value(self.cb_attr_x),
-                                     self.combo_value(self.cb_attr_y))
+            return "{} vs {}".format(self.attr_x.name, self.attr_y.name)
 
     def send_report(self):
-        disc_attr = False
-        if self.data:
-            domain = self.data.domain
-            disc_attr = domain[self.attr_x].is_discrete or \
-                        domain[self.attr_y].is_discrete
+        def name(var):
+            return var and var.name
         caption = report.render_items_vert((
-             ("Color", self.combo_value(self.cb_attr_color)),
-             ("Label", self.combo_value(self.cb_attr_label)),
-             ("Shape", self.combo_value(self.cb_attr_shape)),
-             ("Size", self.combo_value(self.cb_attr_size)),
-             ("Jittering", (self.graph.jitter_continuous or disc_attr) and
-              self.graph.jitter_size)))
+            ("Color", name(self.graph.attr_color)),
+            ("Label", name(self.graph.attr_label)),
+            ("Shape", name(self.graph.attr_shape)),
+            ("Size", name(self.graph.attr_size)),
+            ("Jittering", (self.attr_x.is_discrete or
+                           self.attr_y.is_discrete or
+                           self.graph.jitter_continuous) and
+             self.graph.jitter_size)))
         self.report_plot()
         if caption:
             self.report_caption(caption)
@@ -536,7 +524,7 @@ def test_main(argv=None):
     if len(argv) > 1:
         filename = argv[1]
     else:
-        filename = "iris"
+        filename = "heart_disease"
 
     ow = OWScatterPlot()
     ow.show()
