@@ -5,11 +5,20 @@ import enum
 from xml.sax.saxutils import escape
 from types import SimpleNamespace as namespace
 
+if sys.version_info > (3, 5):
+    from typing import Optional
+
 import numpy
 import sklearn.metrics
 
-from PyQt4 import QtGui, QtCore
-from PyQt4.QtCore import Qt, QEvent, QRectF, QSizeF, pyqtSignal as Signal
+from AnyQt.QtWidgets import (
+    QGraphicsScene, QGraphicsView, QGraphicsWidget, QGraphicsGridLayout,
+    QGraphicsProxyWidget, QGraphicsItemGroup, QGraphicsSimpleTextItem,
+    QGraphicsRectItem, QFrame, QSizePolicy
+)
+from AnyQt.QtGui import QColor, QPen, QBrush, QPainter, QFont, QFontMetrics
+from AnyQt.QtCore import Qt, QEvent, QRectF, QSizeF, QSize, QPointF
+from AnyQt.QtCore import pyqtSignal as Signal
 
 import pyqtgraph as pg
 
@@ -18,6 +27,8 @@ import Orange.distance
 
 from Orange.widgets import widget, gui, settings
 from Orange.widgets.utils import itemmodels
+from Orange.widgets.utils.annotated_data import (create_annotated_table,
+                                                 ANNOTATED_DATA_SIGNAL_NAME)
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.unsupervised.owhierarchicalclustering import \
     WrapperLayoutItem
@@ -34,7 +45,7 @@ class OWSilhouettePlot(widget.OWWidget):
 
     inputs = [("Data", Orange.data.Table, "set_data")]
     outputs = [("Selected Data", Orange.data.Table, widget.Default),
-               ("Other Data", Orange.data.Table)]
+               (ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)]
 
     replaces = [
         "orangecontrib.prototypes.widgets.owsilhouetteplot.OWSilhouettePlot",
@@ -49,7 +60,7 @@ class OWSilhouettePlot(widget.OWWidget):
     cluster_var_idx = settings.ContextSetting(0)
     #: Annotation variable index
     annotation_var_idx = settings.ContextSetting(0)
-    #: Group the silhouettes by cluster
+    #: Group the (displayed) silhouettes by cluster
     group_by_cluster = settings.Setting(True)
     #: A fixed size for an instance bar
     bar_size = settings.Setting(3)
@@ -65,16 +76,30 @@ class OWSilhouettePlot(widget.OWWidget):
 
     class Error(widget.OWWidget.Error):
         need_two_clusters = Msg("Need at least two non-empty clusters")
+        singleton_clusters_all = Msg("All clusters are singletons")
+
+    class Warning(widget.OWWidget.Warning):
+        missing_cluster_assignment = Msg(
+            "{} instance{s} omitted (missing cluster assignment)")
 
     def __init__(self):
         super().__init__()
-
-        self.data = None
-        self._effective_data = None
-        self._matrix = None
-        self._silhouette = None
-        self._labels = None
-        self._silplot = None
+        #: The input data
+        self.data = None         # type: Optional[Orange.data.Table]
+        #: Data after any applied pre-processing step
+        self._effective_data = None  # type: Optional[Orange.data.Table]
+        #: Distance matrix computed from _effective_data
+        self._matrix = None      # type: Optional[Orange.misc.DistMatrix]
+        #: An bool mask (size == len(data)) indicating missing group/cluster
+        #: assignments
+        self._mask = None        # type: Optional[numpy.ndarray]
+        #: An array of cluster/group labels for instances with valid group
+        #: assignment
+        self._labels = None      # type: Optional[numpy.ndarray]
+        #: An array of silhouette scores for instances with valid group
+        #: assignment
+        self._silhouette = None  # type: Optional[numpy.ndarray]
+        self._silplot = None     # type: Optional[SilhouettePlot]
 
         gui.comboBox(
             self.controlArea, self, "distance_idx", box="Distance",
@@ -121,15 +146,15 @@ class OWSilhouettePlot(widget.OWWidget):
         # Ensure that the controlArea is not narrower than buttonsArea
         self.controlArea.layout().addWidget(self.buttonsArea)
 
-        self.scene = QtGui.QGraphicsScene()
-        self.view = QtGui.QGraphicsView(self.scene)
-        self.view.setRenderHint(QtGui.QPainter.Antialiasing, True)
+        self.scene = QGraphicsScene()
+        self.view = QGraphicsView(self.scene)
+        self.view.setRenderHint(QPainter.Antialiasing, True)
         self.view.setAlignment(Qt.AlignTop | Qt.AlignLeft)
         self.mainArea.layout().addWidget(self.view)
 
     def sizeHint(self):
         sh = self.controlArea.sizeHint()
-        return sh.expandedTo(QtCore.QSize(600, 720))
+        return sh.expandedTo(QSize(600, 720))
 
     @check_sql_input
     def set_data(self, data):
@@ -191,11 +216,14 @@ class OWSilhouettePlot(widget.OWWidget):
         self.data = None
         self._effective_data = None
         self._matrix = None
+        self._mask = None
         self._silhouette = None
         self._labels = None
         self.cluster_var_model[:] = []
         self.annotation_var_model[:] = ["None"]
         self._clear_scene()
+        self.Error.clear()
+        self.Warning.clear()
 
     def _clear_scene(self):
         # Clear the graphics scene and associated objects
@@ -210,7 +238,7 @@ class OWSilhouettePlot(widget.OWWidget):
 
     def _invalidate_scores(self):
         # Invalidate and recompute the current silhouette scores.
-        self._labels = self._silhouette = None
+        self._labels = self._silhouette = self._mask = None
         self._update()
         self._replot()
         if self.data is not None:
@@ -219,6 +247,7 @@ class OWSilhouettePlot(widget.OWWidget):
     def _update(self):
         # Update/recompute the distances/scores as required
         if self.data is None:
+            self._mask = None
             self._silhouette = None
             self._labels = None
             self._matrix = None
@@ -231,19 +260,35 @@ class OWSilhouettePlot(widget.OWWidget):
 
         labelvar = self.cluster_var_model[self.cluster_var_idx]
         labels, _ = self.data.get_column_view(labelvar)
+        mask = numpy.isnan(labels)
         labels = labels.astype(int)
-        _, counts = numpy.unique(labels, return_counts=True)
-        if numpy.count_nonzero(counts) >= 2:
-            self.Error.need_two_clusters.clear()
-            silhouette = sklearn.metrics.silhouette_samples(
-                self._matrix, labels, metric="precomputed")
-        else:
-            self.Error.need_two_clusters()
-            labels = silhouette = None
+        labels = labels[~mask]
 
+        labels_unq, counts = numpy.unique(labels, return_counts=True)
+
+        self.Error.singleton_clusters_all.clear()
+        self.Error.need_two_clusters.clear()
+        self.Warning.missing_cluster_assignment.clear()
+
+        if len(labels_unq) < 2:
+            self.Error.need_two_clusters()
+            labels = silhouette = mask = None
+        elif len(labels_unq) == len(labels):
+            self.Error.singleton_clusters_all()
+            labels = silhouette = mask = None
+        else:
+            silhouette = sklearn.metrics.silhouette_samples(
+                self._matrix[~mask, :][:, ~mask], labels, metric="precomputed")
+        self._mask = mask
         self._labels = labels
         self._silhouette = silhouette
 
+        if labels is not None:
+            count_missing = numpy.count_nonzero(mask)
+            if count_missing:
+                self.Warning.missing_cluster_assignment(
+                    count_missing, s="s" if count_missing > 1 else "")
+
     def _set_bar_height(self):
         visible = self.bar_size >= 5
         self._silplot.setBarHeight(self.bar_size)
@@ -275,14 +320,14 @@ class OWSilhouettePlot(widget.OWWidget):
             silplot.selectionChanged.connect(self.commit)
 
             self.scene.setSceneRect(
-                QRectF(QtCore.QPointF(0, 0),
+                QRectF(QPointF(0, 0),
                        self._silplot.effectiveSizeHint(Qt.PreferredSize)))
 
     def _update_bar_size(self):
         if self._silplot is not None:
             self._set_bar_height()
             self.scene.setSceneRect(
-                QRectF(QtCore.QPointF(0, 0),
+                QRectF(QPointF(0, 0),
                        self._silplot.effectiveSizeHint(Qt.PreferredSize)))
 
     def _update_annotations(self):
@@ -296,6 +341,9 @@ class OWSilhouettePlot(widget.OWWidget):
         if self._silplot is not None:
             if annot_var is not None:
                 column, _ = self.data.get_column_view(annot_var)
+                if self._mask is not None:
+                    assert column.shape == self._mask.shape
+                    column = column[~self._mask]
                 self._silplot.setRowNames(
                     [annot_var.str_val(value) for value in column])
             else:
@@ -305,13 +353,23 @@ class OWSilhouettePlot(widget.OWWidget):
         """
         Commit/send the current selection to the output.
         """
-        selected = other = None
+        selected = indices = data = None
         if self.data is not None:
             selectedmask = numpy.full(len(self.data), False, dtype=bool)
             if self._silplot is not None:
                 indices = self._silplot.selection()
+                assert (numpy.diff(indices) > 0).all(), "strictly increasing"
+                if self._mask is not None:
+                    indices = numpy.flatnonzero(~self._mask)[indices]
                 selectedmask[indices] = True
-            scores = self._silhouette
+
+            if self._mask is not None:
+                scores = numpy.full(shape=selectedmask.shape,
+                                    fill_value=numpy.nan)
+                scores[~self._mask] = self._silhouette
+            else:
+                scores = self._silhouette
+
             silhouette_var = None
             if self.add_scores:
                 var = self.cluster_var_model[self.cluster_var_idx]
@@ -321,25 +379,24 @@ class OWSilhouettePlot(widget.OWWidget):
                     self.data.domain.attributes,
                     self.data.domain.class_vars,
                     self.data.domain.metas + (silhouette_var, ))
+                data = self.data.from_table(
+                    domain, self.data)
             else:
                 domain = self.data.domain
+                data = self.data
 
             if numpy.count_nonzero(selectedmask):
                 selected = self.data.from_table(
                     domain, self.data, numpy.flatnonzero(selectedmask))
 
-            if numpy.count_nonzero(~selectedmask):
-                other = self.data.from_table(
-                    domain, self.data, numpy.flatnonzero(~selectedmask))
-
             if self.add_scores:
                 if selected is not None:
                     selected[:, silhouette_var] = numpy.c_[scores[selectedmask]]
-                if other is not None:
-                    other[:, silhouette_var] = numpy.c_[scores[~selectedmask]]
+                data[:, silhouette_var] = numpy.c_[scores]
 
         self.send("Selected Data", selected)
-        self.send("Other Data", other)
+        self.send(ANNOTATED_DATA_SIGNAL_NAME,
+                  create_annotated_table(data, indices))
 
     def send_report(self):
         if not len(self.cluster_var_model):
@@ -363,7 +420,7 @@ class SelectAction(enum.IntEnum):
     NoUpdate, Clear, Select, Deselect, Toogle, Current = 1, 2, 4, 8, 16, 32
 
 
-class SilhouettePlot(QtGui.QGraphicsWidget):
+class SilhouettePlot(QGraphicsWidget):
     """
     A silhouette plot widget.
     """
@@ -379,9 +436,9 @@ class SilhouettePlot(QtGui.QGraphicsWidget):
         self.__selectionRect = None
         self.__selection = numpy.asarray([], dtype=int)
         self.__selstate = None
-        self.__pen = QtGui.QPen(Qt.NoPen)
-        self.__brush = QtGui.QBrush(QtGui.QColor("#3FCFCF"))
-        self.__layout = QtGui.QGraphicsGridLayout()
+        self.__pen = QPen(Qt.NoPen)
+        self.__brush = QBrush(QColor("#3FCFCF"))
+        self.__layout = QGraphicsGridLayout()
         self.__hoveredItem = None
         self.setLayout(self.__layout)
         self.layout().setColumnSpacing(0, 1.)
@@ -414,7 +471,9 @@ class SilhouettePlot(QtGui.QGraphicsWidget):
             raise ValueError("rownames must have the same size as scores")
 
         Ck = numpy.unique(labels)
-        assert Ck[0] >= 0 and Ck[-1] < len(values)
+        if not Ck[0] >= 0 and Ck[-1] < len(values):
+            raise ValueError(
+                "All indices in `labels` must be in `range(len(values))`")
         cluster_indices = [numpy.flatnonzero(labels == i)
                            for i in range(len(values))]
         cluster_indices = [indices[numpy.argsort(scores[indices])[::-1]]
@@ -513,7 +572,7 @@ class SilhouettePlot(QtGui.QGraphicsWidget):
 
         font = self.font()
         font.setPixelSize(self.__barHeight)
-        axispen = QtGui.QPen(Qt.black)
+        axispen = QPen(Qt.black)
 
         ax = pg.AxisItem(parent=self, orientation="top", maxTickLength=7,
                          pen=axispen)
@@ -531,11 +590,11 @@ class SilhouettePlot(QtGui.QGraphicsWidget):
             self.layout().addItem(silhouettegroup, i + 1, 2)
 
             if group.label:
-                line = QtGui.QFrame(frameShape=QtGui.QFrame.VLine)
-                proxy = QtGui.QGraphicsProxyWidget(self)
+                line = QFrame(frameShape=QFrame.VLine)
+                proxy = QGraphicsProxyWidget(self)
                 proxy.setWidget(line)
                 self.layout().addItem(proxy, i + 1, 1)
-                label = QtGui.QGraphicsSimpleTextItem(self)
+                label = QGraphicsSimpleTextItem(self)
                 label.setText("{} ({})".format(escape(group.label),
                                                len(group.scores)))
                 item = WrapperLayoutItem(label, Qt.Vertical, parent=self)
@@ -543,7 +602,7 @@ class SilhouettePlot(QtGui.QGraphicsWidget):
 
             textlist = TextListWidget(self, font=font)
             sp = textlist.sizePolicy()
-            sp.setVerticalPolicy(QtGui.QSizePolicy.Ignored)
+            sp.setVerticalPolicy(QSizePolicy.Ignored)
             textlist.setSizePolicy(sp)
             textlist.setParent(self)
             if group.rownames is not None:
@@ -579,10 +638,10 @@ class SilhouettePlot(QtGui.QGraphicsWidget):
         # Set the current hovered `item` (:class:`QGraphicsRectItem`)
         if self.__hoveredItem is not item:
             if self.__hoveredItem is not None:
-                self.__hoveredItem.setPen(QtGui.QPen(Qt.NoPen))
+                self.__hoveredItem.setPen(QPen(Qt.NoPen))
             self.__hoveredItem = item
             if item is not None:
-                item.setPen(QtGui.QPen(Qt.lightGray))
+                item.setPen(QPen(Qt.lightGray))
 
     def hoverEnterEvent(self, event):
         # Reimplemented
@@ -626,7 +685,7 @@ class SilhouettePlot(QtGui.QGraphicsWidget):
         if event.buttons() & Qt.LeftButton:
             assert self.__selstate is not None
             if self.__selectionRect is None:
-                self.__selectionRect = QtGui.QGraphicsRectItem(self)
+                self.__selectionRect = QGraphicsRectItem(self)
 
             rect = (QRectF(event.buttonDownPos(Qt.LeftButton),
                            event.pos()).normalized())
@@ -798,13 +857,13 @@ class SilhouettePlot(QtGui.QGraphicsWidget):
         return numpy.asarray(self.__selection, dtype=int)
 
 
-class BarPlotItem(QtGui.QGraphicsWidget):
+class BarPlotItem(QGraphicsWidget):
     def __init__(self, parent=None, **kwargs):
         super().__init__(parent, **kwargs)
         self.__barsize = 5
         self.__spacing = 1
-        self.__pen = QtGui.QPen(Qt.NoPen)
-        self.__brush = QtGui.QBrush(QtGui.QColor("#3FCFCF"))
+        self.__pen = QPen(Qt.NoPen)
+        self.__brush = QBrush(QColor("#3FCFCF"))
         self.__range = (0., 1.)
         self.__data = numpy.array([], dtype=float)
         self.__items = []
@@ -842,24 +901,24 @@ class BarPlotItem(QtGui.QGraphicsWidget):
             self.updateGeometry()
 
     def setPen(self, pen):
-        pen = QtGui.QPen(pen)
+        pen = QPen(pen)
         if self.__pen != pen:
             self.__pen = pen
             for item in self.__items:
                 item.setPen(pen)
 
     def pen(self):
-        return QtGui.QPen(self.__pen)
+        return QPen(self.__pen)
 
     def setBrush(self, brush):
-        brush = QtGui.QBrush(brush)
+        brush = QBrush(brush)
         if self.__brush != brush:
             self.__brush = brush
             for item in self.__items:
                 item.setBrush(brush)
 
     def brush(self):
-        return QtGui.QBrush(self.__brush)
+        return QBrush(self.__brush)
 
     def setPlotData(self, values):
         self.__data = numpy.array(values, copy=True)
@@ -886,7 +945,7 @@ class BarPlotItem(QtGui.QGraphicsWidget):
         pen = self.pen()
         brush = self.brush()
         for i in range(self.count()):
-            item = QtGui.QGraphicsRectItem(self)
+            item = QGraphicsRectItem(self)
             item.setPen(pen)
             item.setBrush(brush)
             self.__items.append(item)
@@ -923,17 +982,17 @@ class BarPlotItem(QtGui.QGraphicsWidget):
 from Orange.widgets.visualize.owheatmap import scaled
 
 
-class TextListWidget(QtGui.QGraphicsWidget):
+class TextListWidget(QGraphicsWidget):
     def __init__(self, parent=None, items=None, **kwargs):
         super().__init__(parent, **kwargs)
-        self.setFlag(QtGui.QGraphicsWidget.ItemClipsChildrenToShape, True)
+        self.setFlag(QGraphicsWidget.ItemClipsChildrenToShape, True)
         self.__items = []
         self.__textitems = []
         self.__group = None
         self.__spacing = 0
 
-        sp = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred,
-                               QtGui.QSizePolicy.Preferred)
+        sp = QSizePolicy(QSizePolicy.Preferred,
+                               QSizePolicy.Preferred)
         sp.setWidthForHeight(True)
         self.setSizePolicy(sp)
 
@@ -962,7 +1021,7 @@ class TextListWidget(QtGui.QGraphicsWidget):
         return super().sizeHint(which, constraint)
 
     def __naturalsh(self):
-        fm = QtGui.QFontMetrics(self.font())
+        fm = QFontMetrics(self.font())
         spacing = self.__spacing
         N = len(self.__items)
         width = max((fm.width(text) for text in self.__items),
@@ -988,10 +1047,10 @@ class TextListWidget(QtGui.QGraphicsWidget):
     def __setup(self):
         self.__clear()
         font = self.font()
-        group = QtGui.QGraphicsItemGroup(self)
+        group = QGraphicsItemGroup(self)
 
         for text in self.__items:
-            t = QtGui.QGraphicsSimpleTextItem(text, group)
+            t = QGraphicsSimpleTextItem(text, group)
             t.setData(0, text)
             t.setFont(font)
             t.setToolTip(text)
@@ -1005,7 +1064,7 @@ class TextListWidget(QtGui.QGraphicsWidget):
         if not N:
             return
 
-        fm = QtGui.QFontMetrics(self.font())
+        fm = QFontMetrics(self.font())
         naturalheight = fm.height()
         th = (crect.height() - (N - 1) * spacing) / N
         if th > naturalheight and N > 1:
@@ -1030,7 +1089,8 @@ class TextListWidget(QtGui.QGraphicsWidget):
 
 
 def main(argv=sys.argv):
-    app = QtGui.QApplication(list(argv))
+    from AnyQt.QtWidgets import QApplication
+    app = QApplication(list(argv))
     argv = app.arguments()
     if len(argv) > 1:
         filename = argv[1]
