@@ -1,18 +1,21 @@
 import sys
 import os
 import types
-from functools import reduce
 
-from PyQt4.QtCore import QByteArray, Qt, pyqtSignal as Signal, QSettings, QUrl
-from PyQt4.QtGui import QDialog, QVBoxLayout, QSizePolicy, qApp, QStyle, \
-    QIcon, QShortcut, QKeySequence, QDesktopServices, QSplitter, \
-    QSplitterHandle, QWidget, QPushButton
+from AnyQt.QtWidgets import (
+    QWidget, QDialog, QVBoxLayout, QSizePolicy, QApplication, QStyle,
+    QShortcut, QSplitter, QSplitterHandle, QPushButton
+)
+from AnyQt.QtCore import (
+    Qt, QByteArray, QSettings, QUrl, pyqtSignal as Signal
+)
+from AnyQt.QtGui import QIcon, QKeySequence, QDesktopServices
 
 from Orange.data import FileFormat
 from Orange.widgets import settings, gui
 from Orange.canvas.registry import description as widget_description
 from Orange.canvas.report import Report
-from Orange.widgets.gui import ControlledAttributesDict, notify_changed
+from Orange.widgets.gui import OWComponent
 from Orange.widgets.io import ClipboardFormat
 from Orange.widgets.settings import SettingsHandler
 from Orange.widgets.utils import saveplot, getdeepattr
@@ -47,7 +50,7 @@ class WidgetMetaClass(type(QDialog)):
         from Orange.canvas.registry.description import (
             input_channel_from_args, output_channel_from_args)
 
-        cls = type.__new__(mcs, name, bases, kwargs)
+        cls = super().__new__(mcs, name, bases, kwargs)
         if not cls.name: # not a widget
             return cls
 
@@ -70,8 +73,8 @@ class WidgetMetaClass(type(QDialog)):
         return cls
 
 
-class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
-               metaclass=WidgetMetaClass):
+class OWWidget(QDialog, OWComponent, Report, ProgressBarMixin,
+               WidgetMessagesMixin, metaclass=WidgetMetaClass):
     """Base widget class"""
 
     # Global widget count
@@ -119,9 +122,14 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
     #: Should the widget construct a `controlArea`.
     want_control_area = True
     #: Orientation of the buttonsArea box; valid only if
-    #  `want_control_area` is `True`. Possible values are Qt.Horizontal,
-    #  Qt.Vertical and None for no buttons area
+    #: `want_control_area` is `True`. Possible values are Qt.Horizontal,
+    #: Qt.Vertical and None for no buttons area
     buttons_area_orientation = Qt.Horizontal
+    #: Specify whether the default message bar widget should be created
+    #: and placed into the default layout. If False then clients are
+    #: responsible for displaying messages within the widget in an
+    #: appropriate manner.
+    want_message_bar = True
     #: Widget painted by `Save graph" button
     graph_name = None
     graph_writers = FileFormat.img_writers
@@ -145,6 +153,13 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
     settingsHandler = None
     """:type: SettingsHandler"""
 
+    #: Version of the settings representation
+    #: Subclasses should increase this number when they make breaking
+    #: changes to settings representation (a settings that used to store
+    #: int now stores string) and handle migrations in migrate and
+    #: migrate_context settings.
+    settings_version = 1
+
     savedWidgetGeometry = settings.Setting(None)
 
     #: A list of advice messages (:class:`Message`) to display to the user.
@@ -160,6 +175,7 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
     def __new__(cls, *args, captionTitle=None, **kwargs):
         self = super().__new__(cls, None, cls.get_flags())
         QDialog.__init__(self, None, self.get_flags())
+        OWComponent.__init__(self)
         WidgetMessagesMixin.__init__(self)
 
         stored_settings = kwargs.get('stored_settings', None)
@@ -169,7 +185,6 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
         self.signalManager = kwargs.get('signal_manager', None)
         self.__env = _asmappingproxy(kwargs.get("env", {}))
 
-        setattr(self, gui.CONTROLLED_ATTRIBUTES, ControlledAttributesDict(self))
         self.graphButton = None
         self.report_button = None
 
@@ -271,7 +286,9 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
             self.splitter, margin=4,
             sizePolicy=QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         )
-        self.splitter.setCollapsible(1, False)
+        self.splitter.addWidget(self.mainArea)
+        self.splitter.setCollapsible(
+            self.splitter.indexOf(self.mainArea), False)
         self.mainArea.layout().setContentsMargins(
             0 if self.want_control_area else 4, 4, 4, 4)
 
@@ -289,8 +306,9 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
         """Provide the basic widget layout
 
         Which parts are created is regulated by class attributes
-        `want_main_area`, `want_control_area` and `buttons_area_orientation`,
-        the presence of method `send_report` and attribute `graph_name`.
+        `want_main_area`, `want_control_area`, `want_message_bar` and
+        `buttons_area_orientation`, the presence of method `send_report`
+        and attribute `graph_name`.
         """
         self.setLayout(QVBoxLayout())
         self.layout().setContentsMargins(2, 2, 2, 2)
@@ -299,7 +317,10 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
 
         self.want_main_area = self.want_main_area or self.graph_name
         self._create_default_buttons()
-        self.insert_message_bar()
+
+        if self.want_message_bar:
+            self.insert_message_bar()
+
         self._insert_splitter()
         if self.want_control_area:
             self._insert_control_area()
@@ -345,7 +366,7 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
 
             if restored and not self.windowState() & \
                     (Qt.WindowMaximized | Qt.WindowFullScreen):
-                space = qApp.desktop().availableGeometry(self)
+                space = QApplication.desktop().availableGeometry(self)
                 frame, geometry = self.frameGeometry(), self.geometry()
 
                 #Fix the widget size to fit inside the available space
@@ -449,30 +470,6 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
         if self.signalManager is not None:
             self.signalManager.send(self, signalName, value, id)
 
-    def __setattr__(self, name, value):
-        """Set value to members of this instance or any of its members.
-
-        If member is used in a gui control, notify the control about the change.
-
-        name: name of the member, dot is used for nesting ("graph.point.size").
-        value: value to set to the member.
-        """
-        names = name.rsplit(".")
-        field_name = names.pop()
-        obj = reduce(lambda o, n: getattr(o, n, None), names, self)
-        if obj is None:
-            raise AttributeError("Cannot set '{}' to {} ".format(name, value))
-
-        if obj is self:
-            super().__setattr__(field_name, value)
-        else:
-            setattr(obj, field_name, value)
-
-        notify_changed(obj, field_name, value)
-
-        if self.settingsHandler:
-            self.settingsHandler.fast_save(self, name, value)
-
     def openContext(self, *a):
         """Open a new context corresponding to the given data.
 
@@ -725,6 +722,32 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
 
         self.__msgwidget.accepted.connect(_userconfirmed)
 
+    @classmethod
+    def migrate_settings(cls, settings, version):
+        """Fix settings to work with the current version of widgets
+
+        Parameters
+        ----------
+        settings : dict
+            dict of name - value mappings
+        version : Optional[int]
+            version of the saved settings
+            or None if settings were created before migrations
+        """
+
+    @classmethod
+    def migrate_context(cls, context, version):
+        """Fix contexts to work with the current version of widgets
+
+        Parameters
+        ----------
+        context : Context
+            Context object
+        version : Optional[int]
+            version of the saved context
+            or None if context was created before migrations
+        """
+
 
 class Message(object):
     """
